/* -------------------------------------------------------------------------
   ------------------------------------------------------------------------- */
#ifndef BRD_DIALOGIC_CODE_H
#define BRD_DIALOGIC_CODE_H

#include "brd_generic.h"

#include <process.h>
#include <io.h>
#include <fcntl.h>
#include <sys/stat.h>  // Needed for S_IREAD used in call to dx_fileopen().

// Dialogic header files
// Ensure include path in project options has path to Dialogic 3rd party files
#include <srllib.h>
#include <dxxxlib.h>
#include <dtilib.h>
#include <msilib.h>

typedef struct
{
   DX_IOTT   iott;
   DV_TPT    tpt;
   bool      bPlaying;
   unsigned  position;
} AUDIO_CTRL;

typedef struct
{
	long   evtHandle;	  // Dialogic event handle
	long   evtType;     // Dialogic event code
	long   datalen;     // length of variable event data
	void   *datap;      // pointer to variable event data
} DLG_EVENT;

/* -------------------------------------------------------------------------
   Class:       DialogicBoardEvent

   Purpose:     Implements all the functions defined in the "BoardGeneric"
                class.
   ------------------------------------------------------------------------- */
class DialogicBoardEvent : public BoardEvent
{
	public:
 		DialogicBoardEvent(void);
 		~DialogicBoardEvent(void);

      bool SameEvent(BoardEvent* comparisonEvent);

      DLG_EVENT   DlgEvent;   // structure created to store information
                              // about Dialogic events (see above)
};

/* -------------------------------------------------------------------------
   Class:       BrdDialogic

   Purpose:     Implements all the functions defined in the "BoardGeneric"
                class.
   ------------------------------------------------------------------------- */
class BrdDialogic : public BoardGeneric
{
public:
   BrdDialogic(PIVRINFO pIVRInfo);
   ~BrdDialogic(void);

   IV_STATUS StartReorderToneDetection(void);
   IV_STATUS StopReorderToneDetection(void);
   IV_STATUS Wait(int timeout, bool interruptable);

   // ---- LINE FUNCTIONS -----------------------------------------------------
   virtual IV_STATUS CallStatus(void);

   // flashTime of 0 skips flash
   IV_STATUS FlashHook(long flashTime);

   virtual IV_STATUS Hangup(bool returnHangup = false, char* pszCause = NULL);
   virtual IV_STATUS Pickup(void);
   virtual IV_STATUS PlaceCall(const char* digits, const char* callerID, int numrings,
      callMode mode, int flashdur, bool connectOnCED, bool connectOnSIT, int* pVoiceLength,
      int silenceTimeout, int maxVoiceTimeout,
      ISDNNumberPlan callingNumPlan = UNKNOWN_NUMBER_PLAN,
      ISDNNumberType callingNumType = UNKNOWN_NUMBER_TYPE,
      ISDNNumberPlan calledNumPlan = UNKNOWN_NUMBER_PLAN,
      ISDNNumberType calledNumType = UNKNOWN_NUMBER_TYPE);
   virtual IV_STATUS ReleaseLine(void);
	virtual IV_STATUS RequestLine(int lineNo);
	virtual IV_STATUS Switch(int newPort, bool bListen);
	virtual IV_STATUS UnSwitch(void);
  	virtual IV_STATUS WaitForCall(long timeout, int numrings, bool pickup,
      char* calledDigits, char* callingDigits, char* handsetDisplay, int numdigits = 0);

   // ---- TT FUNCTIONS -------------------------------------------------------
   IV_STATUS FlushDigits(void);
   IV_STATUS GenerateDigits(const char* ttStr);
   IV_STATUS QueryDigit(char& digit);

   // timeout of 0 inidicates returns immediately with IVR_SUCCESS if digit
   // has already occurred or IVR_TIMEOUT if no digit has occurred
   // count must be > 0 or INFINITE
   // digits buffer is populated by GetDigits and the client is repsonible for
   // deleting the buffer
   // Digits A,B,C, and D are in upper case
   IV_STATUS GetDigits(
      unsigned  count,     // Number of digits to collect.
      char*&    digits,    // Digit buffer returned from GetDigits
      int       newTime,   // Timeout between key presses.
      const char* ttOkay,    // Digits that are valid.
      const char* ttDone);   // Digits that terminate TT collection.

   // ---- AUDIO FUNCTIONS ----------------------------------------------------
   IV_STATUS AudioBeep(bool interruptable);

   // maxRecTime of 0 records an empty or short as possible sound file
   // maxRecTime of FOREVER indicates record until interrupt or silence timeout
   IV_STATUS AudioRecord(const char* filename, bool interruptable,
      SOUND_FORMAT format, int volLevel, int maxRecTime, int maxSilence,
      bool disableBeep, bool automaticGainControl);

   // playForTime of 0 indicates do not play file
   // playForTime of FOREVER indicates play until end of file
   IV_STATUS AudioPlay(const char* filename, bool interruptable, SOUND_FORMAT format,
   	int volLevel, bool fancy, unsigned int playFromTime = 0, unsigned int playForTime = FOREVER);

	IV_STATUS AudioPlayBuffer(char* audioBuffer, unsigned bufferSize, bool interruptable,
   	SOUND_FORMAT format, int volLevel);

   IV_STATUS AudioRecognise(int noVoiceTimeout, int endOfVoiceTimeout);
   SOUND_FORMAT GetSRAudioFormat(void);

   IV_STATUS AudioPlayEvalMessage(void);

protected:
   state             myState;
   AUDIO_CTRL        audioCtrl;
 	bool              shutdownRequested;	// To be removed... Information is available in pInfo->shutdownRequested if needed
   bool              safeToShutdown;		// To be removed... Information is available in pInfo->shutdownRequested if needed

   // Prevent compiler from creating default constructor, default copy
   // constructor, and assignment operator.
   BrdDialogic();
   BrdDialogic (const BrdDialogic&);
   BrdDialogic& operator= (BrdDialogic&);

	virtual void InitializeBoard();
   void QueryBoard();
	int  QueryDTIBoard(int bdnum);
	void AddAnalogLines (int voxBdNo, int numChnls, char *VoxBdNm, char nettype);
   void AddDigitalLines (int dtiBdNo, int numLines, char *dtiBdNm, char nettype);
   int  FillAnalogLines (int voxBdNo, int numChnls, char *VoxBdNm);

   // DIALOGIC WIN32 EVENT MODEL - DOES NOT WORK - TF Dec.1/01
   //	void OpenEventQueue();
   // void ShowEvent (DLG_EVENT *eventp);
	IV_STATUS ProcessBoardEvent(BoardEvent* boardEvent, bool& bFound);

   // ---- AUDIO FUNCTIONS ----------------------------------------------------
   // Made virtual for the benefit of UAPI support
   DX_XPB GetEncoding(SOUND_FORMAT format);

   virtual IV_STATUS AudioPlayFile(const char* filename, SOUND_FORMAT format,
      bool interruptable, unsigned int playFromTime = 0, unsigned int playForTime = FOREVER);

   IV_STATUS AudioPlayFileFancy (
      const char* filename,     // name of voice file
      SOUND_FORMAT format);    // the encoding scheme of the file

   void BuildDTMF(const char* sTerm, unsigned short* dTerm);
   bool TermDigit(char dig, const char* ttDone);
   
   virtual void PrintErrorMsg(const char* pszComment, int iChDev);
	virtual IV_STATUS SwitchReset(void);

};

#endif
