/* -------------------------------------------------------------------------
   Name:        IV_DBF2.CPP

   Notes:       - No support for memo fields.
                - No support for indexes.
                - When the functions to create a database are added, it will
                  only succeed if there are *no* memo fields and the file
                  doesn't already exist.
                - Functions need to check to ensure file is open.
                - Not fully tested.  Functions tested so far:
                     [ ] DBFField
                         [X] Constructor
                         [X] Destructor
                         [X] GetName
                         [X] GetValue
                         [X] GetLength
                         [ ] GetDecimal
                         [ ] GetType
                         [ ] SetValue (const char *value)
                         [X] SetValue (const char *value, int offset)
                     [ ] DBFFile
                         [X] Constructor
                         [X] Destructor
                         [X] Open
                         [X] Close
                         [X] Fetch
                         [X] GetFieldValue
                         [ ] Print
                         [ ] IsRecordDeleted
                         [X] GetFieldName
                         [ ] GetFieldType
                         [ ] GetFieldLength
                         [ ] GetFieldDecimal
                         [X] GetFieldValue (int fieldNo)
                         [ ] GetFieldValue (const char *fieldNm)
                         [ ] AddField
                         [ ] Insert
                         [ ] SetFieldValue
                         [ ] Create
                         [ ] SetDate
                         [X] ClearRecord
                         [X] FindField (int fieldNo)
                         [ ] FindField (const char *fieldNm)
   ------------------------------------------------------------------------- */
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <io.h>
#include <time.h>
#include "iv_dbf2.h"

/* *************************************************************************
   **** DBFField Class Definition ******************************************
   ************************************************************************* */

class DBFField : public Link
{
   public:
      DBFField (
         const char *   fieldNm,
         char           type,
         unsigned short len,
         unsigned short dec);
      ~DBFField();
      const char * GetName() { return pszName; }
      const char * GetValue() { return pszValue; }
      char GetType() { return cType; }
      unsigned short GetLength() { return uLen; }
      unsigned short GetDecimal() { return uDec; }
      void SetValue (const char *value);
      void SetValue (const char *value, int offset);

   private:
      unsigned short  uLen;
      unsigned short  uDec;
      char            cType;
      char *          pszName;
      char *          pszValue;
};

/* -------------------------------------------------------------------------
   Name:        DBFField::DBFField                             (Constructor)
   ------------------------------------------------------------------------- */
DBFField::DBFField (
   const char *   fieldNm,
   char           type,
   unsigned short len,
   unsigned short dec) : Link()
{
   pszName = 0;
   pszValue = 0;
   cType = type;
   uLen = len;
   uDec = dec;

   if (fieldNm && *fieldNm) {
      pszName = new char [strlen (fieldNm) + 1];
      strcpy (pszName, fieldNm);
   }
}
/*DBFField::DBFField*/

/* -------------------------------------------------------------------------
   Name:        DBFField::~DBFField                             (Destructor)
   ------------------------------------------------------------------------- */
DBFField::~DBFField()
{
   delete [] pszName;
   delete [] pszValue;
}
/*DBFField::~DBFField*/

/* -------------------------------------------------------------------------
   Name:        DBFField::SetValue
   ------------------------------------------------------------------------- */
void DBFField::SetValue (const char *value)
{
   if (pszValue) {
      delete [] pszValue;
      pszValue = 0;
   }

   if (value && *value) {
      pszValue = new char [strlen (value) + 1];
      strcpy (pszValue, value);
   }
}
/*DBFField::SetValue*/

/* -------------------------------------------------------------------------
   Name:        DBFField::SetValue

   Purpose:     Copies a portion of the string parameter into member
                variable using the length information specified when this
                class was created.

   Notes:       - Need a better description.
                - If we record the amount of memory allocated to this field,
                  it wouldn't be necessary to delete and re-allocate memory.
   ------------------------------------------------------------------------- */
void DBFField::SetValue (const char *value, int offset)
{
   if (pszValue) {
      delete [] pszValue;
      pszValue = 0;
   }

   if (value) {
      pszValue = new char [GetLength() + 1];
      strncpy (pszValue, value + offset, GetLength());
      pszValue[GetLength()] = 0;
   }
}
/*DBFField::SetValue*/


/* *************************************************************************
   **** DBFFile Class Definition *******************************************
   ************************************************************************* */

/* -------------------------------------------------------------------------
   Name:        DBFFile::DBFFile                               (Constructor)
   ------------------------------------------------------------------------- */
DBFFile::DBFFile()
{
   pszRecord = 0;
   fields = new Head();
   fdb = 0;
}
/*DBFFile::DBFFile*/

/* -------------------------------------------------------------------------
   Name:        DBFFile::~DBFFile                               (Destructor)
   ------------------------------------------------------------------------- */
DBFFile::~DBFFile()
{
   Close();
   delete fields;

   // Should be deleted from a Close
   if(pszRecord)
   {
   	delete[] pszRecord;
   }
}
/*DBFFile::~DBFFile*/

/* -------------------------------------------------------------------------
   Name:        DBFFile::Open

   Purpose:     If no fields have been added, this function will attempt to
                open an existing database file.  If fields have been added,
                this function will attempt to create a new database file.
                It will fail if a database file already exists.
   ------------------------------------------------------------------------- */
bool DBFFile::Open (const char *filename)
{
   size_t        readOkay;
   bool          bMoreFields;
   FIELD_REC     fieldRec;
   DBFField *    field;

   if (fdb)
      Close();

   if (!filename || !*filename)
   {
      return false;
   }

   if (!fields->Cardinal())
   {
      fdb = fopen (filename, "rb");      // read-only
      if (!fdb)
      {
         return(false);
      }

      readOkay = fread (&dbfHead, sizeof (DBF_HEAD), 1, fdb);
      if (!readOkay)
      {
         Close();
         return false;
      }
      if ((dbfHead.dbf_id & 0x07) > 3)
      {
         Close();
         return false;
      }
      bMoreFields = true;
      while (bMoreFields) {
         readOkay = fread (&fieldRec, sizeof (FIELD_REC), 1, fdb);
         if (readOkay) {
            bMoreFields = (fieldRec.name[0] != 0x0D);
            if (bMoreFields) {
               if (fieldRec.type != 'N')
                  field = new DBFField (fieldRec.name, fieldRec.type,
                                        fieldRec.len_info.char_len, 0);
               else
                  field = new DBFField (fieldRec.name, fieldRec.type,
                                        fieldRec.len_info.num_size.len,
                                        fieldRec.len_info.num_size.dec);
               fields->AddLast (field);
            }
         } else {
            bMoreFields = false;
         }
      }

      pszRecord = new char [dbfHead.rec_size];
      if (!pszRecord)
      {
         return(false);
      }

      ClearRecord();
      return(true);
   }
   else
   {
      if (access (filename, 00))              // Ensure file doesn't exist.
      {
         if(Create (filename))
         {
            return(true);
         }
         else
         {
            return(false);
         }
      }
   }

   return(false);
}
/*DBFFile::Open*/

/* -------------------------------------------------------------------------
   Name:        DBFFile::Create
   ------------------------------------------------------------------------- */
bool DBFFile::Create (const char *filename)
{
   int           iFieldCnt;
   unsigned int  recSize;
   bool          bIsMemo;
   DBFField *    field;
   FIELD_REC     fieldRec;

   fdb = fopen (filename, "w");
   if (!fdb)
      return false;

   SetDate();
   dbfHead.last_rec = 0L;
   memset (dbfHead.filler, 0, 20);

   iFieldCnt = 0;
   recSize = 0;
   bIsMemo = false;

   field = (DBFField*) fields->First();
   while (field) {
      ++iFieldCnt;
      recSize += field->GetLength();
      if (field->GetType() == 'M')
         bIsMemo = true;
      field = (DBFField*) field->Suc();
   }

   if (bIsMemo)
      // ---- Missing code here. ----
      dbfHead.dbf_id = (char)0x83;
   else
      dbfHead.dbf_id = (char)3;

   dbfHead.rec_size = (unsigned short)(recSize + 1);
   dbfHead.data_offset = (unsigned short)(sizeof(DBF_HEAD) +
                         iFieldCnt * sizeof(FIELD_REC) + 1);

   fwrite (&dbfHead, sizeof (DBF_HEAD), 1, fdb);

   field = (DBFField*) fields->First();
   while (field) {
      memset (fieldRec.name, 0, 11);
      strcpy (fieldRec.name, field->GetName());
      strupr (fieldRec.name);          // dBASE field names must be upper-case.
      fieldRec.type = field->GetType();
      memset (fieldRec.dummy, 0, 4);
      switch (field->GetType()) {
         case 'N':
            fieldRec.len_info.num_size.len = field->GetLength();
            fieldRec.len_info.num_size.dec = field->GetDecimal();
            break;

         default:
            fieldRec.len_info.char_len = field->GetLength();
      }
      memset (fieldRec.filler, 0, 14);
      fwrite (&fieldRec, sizeof (FIELD_REC), 1, fdb);

      field = (DBFField*) field->Suc();
   }
   fwrite ("\015\032", 2, 1, fdb);

   Close();
   return true;
}
/*DBFFile::Create*/

/* -------------------------------------------------------------------------
   Name:        DBFFile::Close

   Purpose:     Closes a file if one is open.  Deletes any field information
                if any have been added.
   ------------------------------------------------------------------------- */
void DBFFile::Close()
{
   if (fdb) {
      fclose (fdb);
      fdb = 0;
   }

   fields->Clear();

   delete [] pszRecord;
   pszRecord = 0;
}
/*DBFFile::Close*/

/* -------------------------------------------------------------------------
   Name:        DBFFile::SetDate
   ------------------------------------------------------------------------- */
void DBFFile::SetDate()
{
   time_t      tloc;
   struct tm * tm;

   tloc = time (&tloc);
   tm = localtime (&tloc);
   dbfHead.last_update[0] = (char) tm->tm_year;
   dbfHead.last_update[1] = (char) (tm->tm_mon + 1);
   dbfHead.last_update[2] = (char) tm->tm_mday;
}
/*DBFFile::SetDate*/

/* -------------------------------------------------------------------------
   Name:        DBFFile::Print

   Purpose:     If a database file has been opened and a record fetched,
                this function will prints out the current record.
   ------------------------------------------------------------------------- */
bool DBFFile::Print()
{
   return false;
}
/*DBFFile::Print*/

/* -------------------------------------------------------------------------
   Name:        DBFFile::ClearRecord

   Purpose:     Prepare the record to receive data.

   Notes:       - DBF records are padded with spaces.
   ------------------------------------------------------------------------- */
void DBFFile::ClearRecord()
{
   if (pszRecord)
      memset (pszRecord, ' ', dbfHead.rec_size);
}
/*DBFFile::ClearRecord*/

/* -------------------------------------------------------------------------
   Name:        DBFFile::IsRecordDeleted

   Purpose:     Checks to see if a record has been deleted.
   ------------------------------------------------------------------------- */
bool DBFFile::IsRecordDeleted()
{
   if (pszRecord)
      return (*pszRecord != ' ');

   return false;
}
/*DBFFile::IsRecordDeleted*/

/* -------------------------------------------------------------------------
   Name:        DBFFile::Fetch
   ------------------------------------------------------------------------- */
bool DBFFile::Fetch (unsigned long recNo)
{
   unsigned long  fPos;
   DBFField *     field;
   unsigned short offset   = 1;

   if (!fdb)
      return false;

   if (recNo < 1 || recNo > dbfHead.last_rec)
      return false;

   ClearRecord();

   // Need to implement error checking.
   fPos = dbfHead.data_offset +
          ((recNo - 1) * dbfHead.rec_size);
   if (fseek (fdb, fPos, SEEK_SET))
      return false;

   if (fread (pszRecord, dbfHead.rec_size, 1, fdb) != 1)
      return false;

   field = (DBFField*) fields->First();
   while (field) {
      field->SetValue (pszRecord, offset);
      offset += field->GetLength();
      field = (DBFField*) field->Suc();
   }

   return true;
}
/*DBFFile::Fetch*/

/* -------------------------------------------------------------------------
   Name:        DBFFile::FindField
   ------------------------------------------------------------------------- */
DBFField * DBFFile::FindField (int fieldNo)
{
   int         i;
   DBFField *  field;

   if (fieldNo <= 0)
      return 0;

   i = 1;
   field = (DBFField*) fields->First();
   while (field) {
      if (i == fieldNo)
         return field;
      field = (DBFField*) field->Suc();
      ++i;
   }
   return 0;
}
/*DBFFile::FindField*/

/* -------------------------------------------------------------------------
   Name:        DBFFile::FindField
   ------------------------------------------------------------------------- */
DBFField * DBFFile::FindField (const char *fieldNm)
{
   DBFField *  field;

   if (!fieldNm || !*fieldNm)
      return 0;

   field = (DBFField*) fields->First();
   while (field) {
      if (!strcmp (field->GetName(), fieldNm))
         return field;
      field = (DBFField*) field->Suc();
   }
   return 0;
}
/*DBFFile::FindField*/

/* -------------------------------------------------------------------------
   Name:        DBFFile::GetFieldName
   ------------------------------------------------------------------------- */
/*
char* DBFFile::GetFieldName(char* fieldName, int fieldNo)
{
   DBFField*  field;

   if((field = FindField (fieldNo)) != NULL)
   {
      strcpy(fieldName, field->GetName());
   }
   else
   {
   	*fieldName = '\0';
   }

   return(fieldName);
}
*/
/*DBFFile::GetFieldName*/

/* -------------------------------------------------------------------------
   Name:        DBFFile::GetFieldType
   ------------------------------------------------------------------------- */
char DBFFile::GetFieldType (int fieldNo)
{
   DBFField *  field;

   if ((field = FindField (fieldNo)) != NULL)
      return field->GetType();

   return 0;
}
/*DBFFile::GetFieldType*/

/* -------------------------------------------------------------------------
   Name:        DBFFile::GetFieldLength
   ------------------------------------------------------------------------- */
unsigned short DBFFile::GetFieldLength (int fieldNo)
{
   DBFField *  field;

   if ((field = FindField (fieldNo)) != NULL)
      return field->GetLength();

   return 0;
}
/*DBFFile::GetFieldLength*/

/* -------------------------------------------------------------------------
   Name:        DBFFile::GetFieldDecimal
   ------------------------------------------------------------------------- */
unsigned short DBFFile::GetFieldDecimal (int fieldNo)
{
   DBFField *  field;

   if ((field = FindField (fieldNo)) != NULL)
      return field->GetDecimal();

   return 0;
}
/*DBFFile::GetFieldDecimal*/

int DBFFile::GetFieldValueLength(int fieldNo)
{
   DBFField*  field;

   if ((field = FindField (fieldNo)) != NULL)
   {
      return(strlen(field->GetValue()));
   }
   else
   {
   	return(0);
   }
}

/* -------------------------------------------------------------------------
   Name:        DBFFile::GetFieldValue
   ------------------------------------------------------------------------- */
char* DBFFile::GetFieldValue(char* fieldValue, int fieldNo)
{
   DBFField*  field;

   if ((field = FindField (fieldNo)) != NULL)
   {
      strcpy(fieldValue, field->GetValue());
   }
   else
   {
   	*fieldValue = '\0';
   }

   return(fieldValue);
}
/*DBFFile::GetFieldValue*/

int DBFFile::GetFieldValueLength(const char* fieldNm)
{
   DBFField*  field;

   if((field = FindField (fieldNm)) != NULL)
   {
      return(strlen(field->GetValue()));
   }
   else
   {
   	return(0);
   }
}

/* -------------------------------------------------------------------------
   Name:        DBFFile::GetFieldValue
   ------------------------------------------------------------------------- */
char* DBFFile::GetFieldValue(char* fieldValue, const char* fieldNm)
{
   DBFField*  field;

   if((field = FindField (fieldNm)) != NULL)
   {
      strcpy(fieldValue, field->GetValue());
   }
   else
   {
   	*fieldValue = '\0';
   }

   return(fieldValue);
}
/*DBFFile::GetFieldValue*/

/* -------------------------------------------------------------------------
   Name:        DBFFile::AddField

   Notes:       - Fields can only be added for new tables.  This function
                  will fail if a database is open.
   ------------------------------------------------------------------------- */
bool DBFFile::AddField (
   const char *   /*fieldNm*/,
   char           /*type*/,
   unsigned short /*len*/,
   unsigned short /*dec*/)
{
   return false;
}
/*DBFFile::AddField*/
