#include <algorithm>
using std::transform;

#include "step_RegistryRead.h"

StepRegistryRead::StepRegistryRead(STEPREC& stepRecord) : Step(stepRecord)
{
   variable = stepRecord.s1.arr;
   regKey = stepRecord.s2.arr;
   regValue = stepRecord.s3.arr;
}

StepRegistryRead::~StepRegistryRead(void)
{
}

const char* StepRegistryRead::GetStepName(void) const
{
	return("Read Registry");
}

IV_STATUS StepRegistryRead::Execute(PIVRINFO pInfo)
{
   // Evaluate all parameters before exiting
   bool bParmsValid = true;
   char* value;
   string parsedRegKey;
   string parsedRegValue;
   HKEY hRegHandle;
   HKEY hkey;
   DWORD dwType;
   DWORD dwLen;
   LONG result = ERROR_SUCCESS;
   string sData;

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

   // Validate TeleFlow variable to receive registry value
   if(!pInfo->variables->IsVariableFormat(variable.c_str()))
   {
      pInfo->ivLog->Print("      ERROR: Must supply a variable name to receive registry value\n");
      bParmsValid = false;
   }

   // Validate registry key name
   if((value = pInfo->variables->ParseParameter("Registry key name", regKey.c_str())) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      parsedRegKey = value;
      delete[] value;

      // convert to upper case
      transform(parsedRegKey.begin(), parsedRegKey.end(), parsedRegKey.begin(), toupper);

      // Key must start with HKEY_CLASSES_ROOT, HKEY_CURRENT_USER, HKEY_LOCAL_MACHINE, HKEY_USERS, HKEY_CURRENT_CONFIG
      if(parsedRegKey.find("HKEY_CLASSES_ROOT") == 0)
      {
         hRegHandle = HKEY_CLASSES_ROOT;
      }
      else if(parsedRegKey.find("HKEY_CURRENT_USER") == 0)
      {
         hRegHandle = HKEY_CURRENT_USER;
      }
      else if(parsedRegKey.find("HKEY_LOCAL_MACHINE") == 0)
      {
         hRegHandle = HKEY_LOCAL_MACHINE;
      }
      else if(parsedRegKey.find("HKEY_USERS") == 0)
      {
         hRegHandle = HKEY_USERS;
      }
      else if(parsedRegKey.find("HKEY_CURRENT_CONFIG") == 0)
      {
         hRegHandle = HKEY_CURRENT_CONFIG;
      }
      else
      {
         pInfo->ivLog->Print("      ERROR: Invalid Windows registry key specified\n");
         bParmsValid = false;
      }
   }

   // Validate registry value name
   if((value = pInfo->variables->ParseParameter("Registry value name", regValue.c_str())) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      parsedRegValue = value;
      delete[] value;
   }

   if(!bParmsValid)
   {
      return(IVR_FATAL);
   }

   pInfo->ivLog->Print("      %s = %s\\%s\n", variable.c_str(), parsedRegKey.c_str(), parsedRegValue.c_str());

   // remove the "root" key handle identifier
   parsedRegKey = parsedRegKey.substr(parsedRegKey.find("\\")+1);

   // read the key from the registry
   result = RegOpenKeyEx(hRegHandle, parsedRegKey.c_str(), 0, KEY_READ, &hkey);
   if(result == ERROR_SUCCESS)
   {
      result = RegQueryValueEx(hkey, parsedRegValue.c_str(), NULL, &dwType, NULL, &dwLen);
      if(result == ERROR_SUCCESS)
      {
         switch(dwType)
         {
            case REG_NONE:
            {
               sData = "";
               break;
            }

            case REG_DWORD:
            {
               DWORD   dwData;
               char number[16];
               result = RegQueryValueEx(hkey, parsedRegValue.c_str(), NULL, &dwType, (LPBYTE) &dwData, &dwLen);
               sprintf(number, "%i", dwData);
               sData = number;
               break;
            }

            case REG_SZ:
            case REG_EXPAND_SZ:
            case REG_MULTI_SZ:
            {
               value = new char[dwLen+1];
               result = RegQueryValueEx(hkey, parsedRegValue.c_str(), NULL, &dwType, (LPBYTE) value, &dwLen);
               sData = value;
               delete[] value;
               break;
            }

            case REG_BINARY:
            {
               pInfo->ivLog->Print("      Binary registry data not supported by TeleFlow\n");
               return(IVR_NOTSUPPORTED);
            }
            case REG_DWORD_BIG_ENDIAN:
            {
               pInfo->ivLog->Print("      Big-endian registry data not supported by TeleFlow\n");
               return(IVR_NOTSUPPORTED);
            }
            case REG_LINK:
            {
               pInfo->ivLog->Print("      Link registry data not supported by TeleFlow\n");
               return(IVR_NOTSUPPORTED);
            }
            case REG_RESOURCE_LIST:
            {
               pInfo->ivLog->Print("      Resource List registry data not supported by TeleFlow\n");
               return(IVR_NOTSUPPORTED);
            }

            default:
            {
               pInfo->ivLog->Print("      Unknown registry data type, 0x%x\n", dwType);
               return(IVR_FAIL);
            }
         }
      }
      else
      {
         if(result == ERROR_FILE_NOT_FOUND)
         {
            pInfo->ivLog->Print("      Specified value not found\n");
            return(IVR_FAIL);
         }
      }
   }
   else
   {
      if(result == ERROR_FILE_NOT_FOUND)
      {
         pInfo->ivLog->Print("      Specified key not found\n");
         return(IVR_FAIL);
      }
   }

   if(result != ERROR_SUCCESS)
   {
      char MsgBuf[1024];

      FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM,
                    0,
                    result,
                    MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT),
                    MsgBuf,
                    1024,
                    0
                   );

      pInfo->ivLog->Print("      %s\n", MsgBuf);
      return(IVR_FAIL);
   }

   // Save data to TeleFlow variable specified
   if(!pInfo->variables->Set(variable.c_str(), sData.c_str()))
   {
      pInfo->ivLog->Print ("      ERROR: Failed to set variable '%s'\n", variable.c_str());
      return(IVR_FATAL);
   }

   return(IVR_SUCCESS);
}
