#include <stdlib.h>               // atoi
#include "step_dictate.h"
#include "..\escutil\iv_str.h"	// strtrim
#include "..\BoardDLLs\brd_generic.h"

StepDictate::StepDictate(STEPREC& stepRecord) : Step(stepRecord)
{
   TTActionsString = stepRecord.s1.arr;
   silenceTimeout = stepRecord.s2.arr;
   exitPositionVariable = stepRecord.s3.arr;
   maximumRecordTime = stepRecord.s4.arr;
   startPosition = stepRecord.s5.arr;
   dictationFileName = stepRecord.s6.arr;
   menuMessageCode = stepRecord.s7.arr;
   menuFileName = stepRecord.s9.arr;
   menuTimeout = stepRecord.s8.arr;
   menuRetries = stepRecord.s10.arr;
   startBehaviour = (START_BEHAVIOUR) stepRecord.u1;
   // Default value of 0 indicates AGC should be used (i.e. true)
   automaticGainControl = !((bool) stepRecord.u2);
   dictationFormat = (SOUND_FORMAT) stepRecord.u3;
   disableBeep = (bool) stepRecord.u4;
   endBehaviour = (END_BEHAVIOUR) stepRecord.u6;
   menuFormat = (SOUND_FORMAT) stepRecord.u3;

   // initialize info pointers (char*) in TTActions array
   for(unsigned int index = 0; index < 12; index++)
   {
      TTActions[index].info = NULL;
   }
}

StepDictate::~StepDictate(void)
{
   for(unsigned int index = 0; index < 12; index++)
   {
      delete[] TTActions[index].info;
   }
}

const char* StepDictate::GetStepName(void) const
{
	return("Dictate");
}

StepDictate::DICTATE_ACTION StepDictate::TranslateAction(const char* action)
{
   if((action == NULL) || (action[0] == '\0') || (strcmp(action, "EXIT") == 0))
   {
      return(ACTION_EXIT);
   }
   else if(strcmp(action, "RWBEG") == 0)
   {
      return(ACTION_PLAY_FROM_BEGINNING);
   }
   else if(strcmp(action, "RW") == 0)
   {
      return(ACTION_REWIND_AND_PLAY);
   }
   else if(strcmp(action, "PLAY") == 0)
   {
      return(ACTION_PLAY_START_PAUSE);
   }
   else if(strcmp(action, "REC") == 0)
   {
      return(ACTION_RECORD_START_PAUSE);
   }
   else if(strcmp(action, "FF") == 0)
   {
      return(ACTION_FAST_FORWARD_AND_PLAY);
   }
   else if(strcmp(action, "FFEND") == 0)
   {
      return(ACTION_FAST_FORWARD_TO_END);
   }
   else if(strcmp(action, "FFEND") == 0)
   {
      return(ACTION_FAST_FORWARD_TO_END);
   }
   else if(strcmp(action, "NA") == 0)
   {
      return(ACTION_NO_ACTION);
   }
   else if(strcmp(action, "DONE") == 0)
   {
      return(ACTION_END_RECORDING);
   }
   else if(strcmp(action, "PLAYMENU") == 0)
   {
      return(ACTION_PLAY_MENU);
   }
   else
   {
      return(ACTION_INVALID);
   }
}

const char* StepDictate::ActionName(StepDictate::DICTATE_ACTION action)
{
   switch(action)
   {
      case ACTION_EXIT:                   return("Exit");
      case ACTION_PLAY_FROM_BEGINNING:    return("Play from beginning");
      case ACTION_REWIND_AND_PLAY:        return("Rewind and play");
      case ACTION_PLAY_START_PAUSE:       return("Play/pause");
      case ACTION_RECORD_START_PAUSE:     return("Record/pause");
      case ACTION_FAST_FORWARD_AND_PLAY:  return("Fast forward and play");
      case ACTION_FAST_FORWARD_TO_END:    return("Fast forward to end");
      case ACTION_NO_ACTION:              return("No action");
      case ACTION_END_RECORDING:          return("End recording");
      case ACTION_PLAY_MENU:              return("Play menu");
      case ACTION_INVALID:
      default:                            return("Invalid");
   }
}

const char* StepDictate::StartBehaviourName(START_BEHAVIOUR behaviour)
{
   switch(behaviour)
   {
      case BEHAVIOUR_RECORD_FROM_BEGINNING:  return("recording from beginning");
      case BEHAVIOUR_RECORD_FROM_POSITION:   return("recording from position");
      case BEHAVIOUR_PLAY_FROM_BEGINNING:    return("playing from beginning");
      case BEHAVIOUR_PLAY_FROM_POSITION:     return("playing from position");
      case BEHAVIOUR_RECORD_FROM_END:        return("recording from end");
      default:                               return("invalid behaviour");
   }
}

const char* StepDictate::EndBehaviourName(END_BEHAVIOUR behaviour)
{
   switch(behaviour)
   {
      case BEHAVIOUR_EXIT:       return("Exit");
      case BEHAVIOUR_PLAY_MENU:  return("Play menu");
      default:                   return("Invalid behaviour");
   }
}

// -------------------------------------------------------------------------
//   Name:        StepDictate::Execute
//
//   Notes:       - I don't think this function should use the global timeout
//                  default.  I think if an invalid time is set, it should
//                  default to forever.
// -------------------------------------------------------------------------
IV_STATUS StepDictate::Execute(PIVRINFO pInfo)
{
   char* value;
   bool bParmsValid = true;
   int iSilenceTimeout;
   int iMaximumRecordTime;
   std::streamoff iStartPosition;
   int volLevel;
   int iMenuTimeout;
   int iMenuRetries;
   string fileName;
   char* dictationPathName;
   char* menuPathName;
   IV_STATUS status;
   std::streamoff exitPosition;

   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

	if(PortRequired(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FATAL);
   }

	// Flush TT buffer so that we don't accidentally interrupt record before it starts
   pInfo->ivBoard->FlushDigits();

   for(int index = 0; index < 12; index++)
   {
      char TT;

      // Get action
      value = FindDelimitedElement(TTActionsString.c_str(), 0, 2*index + 1, ",", true, false);
      TTActions[index].action = TranslateAction(value);

      // Get info
      TTActions[index].info = FindDelimitedElement(TTActionsString.c_str(), 0, 2*index + 2, ",", true, false);

      switch(index)
      {
         // same order as telephone keypad
         case 9: TT = '*'; break;
         case 10: TT = '0'; break;
         case 11: TT = '#'; break;
         default: TT = (char) ('1' + index);
      }

      pInfo->ivLog->Print("      %c: %s", TT, ActionName(TTActions[index].action));
      if((TTActions[index].action == ACTION_REWIND_AND_PLAY) || (TTActions[index].action == ACTION_FAST_FORWARD_AND_PLAY))
      {
         pInfo->ivLog->Print(" (%s ms)", TTActions[index].info);
      }
      pInfo->ivLog->Print("\n");

      if(TTActions[index].action == ACTION_INVALID)
      {
         pInfo->ivLog->Print("      ERROR: Action '%s' for TT '%c' is undefined\n", value, TT);
         bParmsValid = false;
      }

      delete[] value;
   }

   if((value = pInfo->variables->ParseParameter("Silence TIMEOUT", silenceTimeout.c_str(), "@TIMEOUT")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      iSilenceTimeout = atoi(value);  // iSilenceTimeout is in ms
      delete[] value;

      if(iSilenceTimeout == 0)
      {
         pInfo->ivLog->Print("      ERROR: Silence TIMEOUT '%s' cannot be 0\n",
         pInfo->variables->IsParsedEmpty(silenceTimeout.c_str()) ? "@TIMEOUT" : silenceTimeout.c_str());
         bParmsValid = false;
      }
   }

   if((value = pInfo->variables->ParseParameter("Maximum record time", maximumRecordTime.c_str(),
      "-1", "Empty Maximum record time defaults to none")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      iMaximumRecordTime = atoi(value);  // iMaxTime is in milliseconds.
      delete[] value;
   }

   if((value = pInfo->variables->ParseParameter("Start Position", startPosition.c_str(), "0")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      iStartPosition = atoi(value);  // iStartPosition is in milliseconds.
      delete[] value;

      if(iStartPosition < 0)
      {
         pInfo->ivLog->Print("      WARNING: Record/Play from position '%s' evaluates to < 0 and is changed to 0\n", startPosition.c_str());
         iStartPosition = 0;
      }
   }

   if((value = pInfo->variables->ParseParameter("Voice file", dictationFileName.c_str())) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      strtrim(value);
      dictationPathName = pInfo->variables->FilePath(value);
      delete[] value;
   }

   if(endBehaviour == BEHAVIOUR_PLAY_MENU)
   {
      if((value = pInfo->variables->ParseParameter("Menu timeout", menuTimeout.c_str(), "@TIMEOUT")) == NULL)
      {
         bParmsValid = false;
      }
      else
      {
         iMenuTimeout = atoi(value);  // iMenuTimeout is in milliseconds.
         delete[] value;

         if(iMenuTimeout == 0)
         {
            pInfo->ivLog->Print("      ERROR: Menu timeout '%s' cannot be 0\n",
            pInfo->variables->IsParsedEmpty(menuTimeout.c_str()) ? "@TIMEOUT" : menuTimeout.c_str());
            bParmsValid = false;
         }
      }

      if((value = pInfo->variables->ParseParameter("Menu retries", menuRetries.c_str(), "2")) == NULL)
      {
         bParmsValid = false;
      }
      else
      {
         iMenuRetries = atoi(value);
         delete[] value;
      }

      if(!menuMessageCode.empty())
      {
         if((value = pInfo->variables->ParseParameter("Menu Message Code", menuMessageCode.c_str(), "")) == NULL)
         {
            bParmsValid = false;
         }
         else
         {
            if(strlen(value) == 0)
            {
               pInfo->ivLog->Print("      WARNING: Menu Message Code '%s' evaluates empty\n", menuMessageCode.c_str());
               pInfo->ivLog->Print("         No menu will be played\n");
            }
            else
            {
               char* scriptCodeFileName;
               SOUND_FORMAT format;

               if(GetScriptCodeFileNameAndFormat(pInfo, value, scriptCodeFileName, format) != IVR_SUCCESS)
               {
                  bParmsValid = false;
               }
               else
               {
                  fileName = scriptCodeFileName;
                  delete[] scriptCodeFileName;
               }
            }

            delete[] value;
         }
      }
      else if(!menuFileName.empty())
      {
         pInfo->ivLog->Print("      Menu Voice File: %s\n", menuFileName.c_str());

         if(pInfo->variables->IsParsedEmpty(menuFileName.c_str()))
         {
            pInfo->ivLog->Print("      WARNING: Menu Voice File '%s' undefined\n", menuFileName.c_str());
            pInfo->ivLog->Print("         No menu will be played\n");
         }
         else
         {
            value = pInfo->variables->Parse(menuFileName.c_str());
            fileName = value;
            delete[] value;
         }
      }
   }
   else
   {
      iMenuTimeout = 0;
   }

   if(!bParmsValid)
   {
      return(IVR_FATAL);
   }

   if(!fileName.empty())
   {
      menuPathName = pInfo->variables->FilePath(fileName.c_str());
   }
   else
   {
      menuPathName = NULL;
   }

   pInfo->ivLog->Print("      Start by %s", StartBehaviourName(startBehaviour));
   if((startBehaviour == BEHAVIOUR_RECORD_FROM_POSITION) || (startBehaviour == BEHAVIOUR_PLAY_FROM_POSITION))
   {
      pInfo->ivLog->Print(" (%d)", iStartPosition);
   }
   pInfo->ivLog->Print("\n");
   pInfo->ivLog->Print("      %s at end of audio", EndBehaviourName(endBehaviour));
   if(endBehaviour == BEHAVIOUR_PLAY_MENU)
   {
      pInfo->ivLog->Print(" and repeat menu every %d ms up to %d times", iMenuTimeout, iMenuRetries);
   }
   pInfo->ivLog->Print("\n");

   // Get volume information
   volLevel = atoi(pInfo->variables->Get("@VOLUME"));

   if(pInfo->portLicense == UNLICENSED)
   {
      pInfo->ivLog->Print ("      THIS PORT IS UNLICENSED, RECORDING AT MOST 60 SECONDS\n");
      // Record for at most 60 seconds
      iMaximumRecordTime = (iMaximumRecordTime < 60000) && (iMaximumRecordTime != FOREVER) ? iMaximumRecordTime : 60000;
   }

   pInfo->ivLog->Print("      Dictating '%s'\n", dictationPathName);

   status = pInfo->ivBoard->Dictate(TTActions, iSilenceTimeout, exitPosition,
      iMaximumRecordTime, iStartPosition, dictationPathName, startBehaviour,
      automaticGainControl, dictationFormat, disableBeep, endBehaviour, volLevel,
      menuPathName, menuFormat, iMenuTimeout, iMenuRetries);

   delete[] dictationPathName;
   delete[] menuPathName;

   if(!exitPositionVariable.empty())
   {
      char number[14];
      sprintf(number, "%d", exitPosition);
      if(!pInfo->variables->Set(exitPositionVariable.c_str(), number))
      {
         return(IVR_FATAL);
      }
   }

   return(status);
}
