#include "step_xmlfetch.h"

StepXMLFetch::StepXMLFetch(STEPREC& stepRecord) : Step(stepRecord)
{
   handle = stepRecord.s1.arr;
   node = stepRecord.s2.arr;
   valueVar = stepRecord.s3.arr;
   attributes = stepRecord.s4.arr;
   iScope = (XMLFetch::SCOPE) stepRecord.u1;
   iDirection = (FETCH_DIRECTION) stepRecord.u2;
}

StepXMLFetch::~StepXMLFetch(void)
{
}

const char* StepXMLFetch::GetStepName(void) const
{
	return("XML Fetch");
}

// -------------------------------------------------------------------------
//   Name:        StepXMLFetch::Execute                               (Public)
//
//   Purpose:     Search for an XML node and capture the data to
//                TeleFlow variables.
//
//   Modified:    May 15/02 - TF: Created
//                Jul.22/02 - TF: Added fetch direction
// -------------------------------------------------------------------------
IV_STATUS StepXMLFetch::Execute (PIVRINFO pInfo)
{
   IV_STATUS  status;           // Value to be returned
   char*      value;
   // Evaluate all parameters before exiting
   bool bParmsValid = true;
   int        iHandle;          // Local integer copy of xml handle
   string     parsedNode;       // Local text buffer to contain node specification
   string     parsedAttributes; // Local text buffer to contain attribute assignments
   XML*       pxmlReq;          // pointer to TeleFlow XML object
   XMLFetch*  pxFetch;          // pointer to TeleFlow XML Fetch object


   if(Status(pInfo) != IVR_SUCCESS)
   {
      return(IVR_FAIL);
   }

   // --- Retrieve handle number
   if((value = pInfo->variables->ParseParameter("Handle", handle.c_str())) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      iHandle = atoi(value);
      delete[] value;
   }

   // --- Get the Node that the user is parsing for
   if((value = pInfo->variables->ParseParameter("Node", node.c_str())) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      parsedNode = value;
      delete[] value;
   }

   // --- Get the attribute assignments
   if((value = pInfo->variables->ParseParameter("Attributes", attributes.c_str(), "")) == NULL)
   {
      bParmsValid = false;
   }
   else
   {
      parsedAttributes = value;
      delete[] value;
   }

   if(!bParmsValid)
   {
      return(IVR_FATAL);
   }

   // pInfo->ivLog->Print("      Attribute assignments\n");
   // pInfo->ivLog->Print("      ---------------------\n");
   // pInfo->ivLog->Print("%s\n", parsedAttributes.c_str());
   // pInfo->ivLog->Print("      ---------------------\n");

   // --- Validate the variable specified to receive node's value
   if(valueVar.empty())
   {
      pInfo->ivLog->Print("      Warning: No variable specified to capture node value\n");
   }
   else if(!pInfo->variables->IsVariableFormat(valueVar.c_str()))
   {
      pInfo->ivLog->Print("      ERROR: '%s' is not a valid TeleFlow variable name\n", valueVar.c_str());
   }
   else
   {
      pInfo->ivLog->Print("      Save node value to '%s'\n", valueVar.c_str());
   }

   // --- Display variable creation scope
   switch (iScope)
   {
      case XMLFetch::DEFAULT:
         pInfo->ivLog->Print("      Variable assignment scope: DEFAULT\n");
         break;
      case XMLFetch::LOCAL:
         pInfo->ivLog->Print("      Variable assignment scope: LOCAL\n");
         break;
      case XMLFetch::GLOBAL:
         pInfo->ivLog->Print("      Variable assignment scope: GLOBAL\n");
         break;
      default:
         pInfo->ivLog->Print("      Variable assignment scope: UNKNOWN!\n");
         pInfo->ivLog->Print("      ERROR: Invalid variable creation scope %i\n", iScope);
         return(IVR_FATAL);
   } // end switch

   // --- Validate search direction
   if(iDirection != BACKWARD)
   {
      iDirection = FORWARD;      // Forward fetch is the default
   }

   // ---
   // --- Find the XML object according to handle requested
   // ---
   pInfo->ivLog->Print("      Handle %i: Retrieving node '%s'\n", iHandle, parsedNode.c_str());

   pxmlReq = pInfo->xmlList->FindXML (iHandle);
   if (pxmlReq == NULL)
   {
      pInfo->ivLog->Print("      ERROR: Handle '%i' not open.\n", iHandle);
      return IVR_FATAL;
   }

   // ---
   // --- Find/Create the Fetch object according to context specified (parsedNode)
   // ---

   pxFetch = pxmlReq->xFetchList->FindFetch(parsedNode.c_str());
   if (pxFetch != NULL)
   {
      // --- Fetch object already open, search within it
      if (iDirection == FORWARD)
      {
         status = pxFetch->FindNext();    // find next node
      }
      else
      {
         status = pxFetch->FindPrev();    // find previous node
      }
   }
   else
   {
      // --- Fetch not open yet; create a new XMLFetch for this node (parsedNode)
      pxFetch = new XMLFetch(pInfo->ivLog, parsedNode.c_str());
      if (pxFetch == NULL)
      {
         pInfo->ivLog->Print("      ERROR: Out of memory.\n");
         return IVR_FATAL;
      }
      // --- Add new fetch object to the linked list for this XML object
      pxmlReq->xFetchList->AddLast ((Link*)pxFetch);

      // --- Find the requested node
      if (iDirection == FORWARD)
      {
         // --- Setup the search for the first time
         status = pxFetch->FindFirst(pxmlReq->xmlDoc);
      }
      else
      {
         // --- Cannot fetch backward until we get past the first occurrence
         status = IVR_FAIL;
      }
   }

   if(status == IVR_SUCCESS)
   {
      // --- We are on the node, now copy the data into TeleFlow variables
      status  = pxFetch->CaptureData(pInfo, valueVar.c_str(), parsedAttributes.c_str(), iScope);
   }

   return(status);
}
// StepXMLFetch::Execute

#undef FETCH_FORWARD
#undef FETCH_BACKWARD

