/* -------------------------------------------------------------------------
   File:        GUI_LOG.CPP

                Copyright (c) 1997 - 2001 enGenic.
                All rights reserved.
   ------------------------------------------------------------------------- */
#include <stdarg.h>
#include <string.h>
#include <sys/stat.h>

#include "..\TFServer\ivr_defs.h"

#include "gui_log.h"

/* -------------------------------------------------------------------------
   Name:        GetFileSize
   ------------------------------------------------------------------------- */
static long GetFileSize (FILE *fp)
{
   struct stat  statbuf;

   if (fp && fstat (fileno(fp), &statbuf) == 0)
   {
      return statbuf.st_size;
   }
   return -1;
}
/*GetFileSize*/

/* -------------------------------------------------------------------------
   Name:        GUILog::GUILog
   ------------------------------------------------------------------------- */
GUILog::GUILog(int maxFileSize, unsigned int maxLines, unsigned int maxLineSize)
 : Log(maxFileSize)
{
   if(maxLines < 50)
   {
      iMaxLines = 50;
   }
   else
   {
      iMaxLines = maxLines;
   }

   if(maxLineSize < 80)
   {
      iMaxLineSize = 80;
   }
   else
   {
      iMaxLineSize = maxLineSize;
   }

   lines.resize(iMaxLines);

   iFirstLine    = 0;
   iLastLine     = 0;

   pFnPrint = NULL;
}
/*GUILog::GUILog*/

/* -------------------------------------------------------------------------
   Name:        GUILog::~GUILog
   ------------------------------------------------------------------------- */
GUILog::~GUILog(void)
{
}
/*GUILog::~GUILog*/

/* -------------------------------------------------------------------------
   Name:        GUILog::SetPrintFnPtr
   ------------------------------------------------------------------------- */
void GUILog::SetPrintFnPtr (PrintFnPtr fn, void *param)
{
   pFnPrint = fn;
   pFnPrintParam = param;
}
/*GUILog::SetPrintFnPtr*/

// -------------------------------------------------------------------------
// Name:        GUILog::GetLines
//
// Purpose:     Copies lines from the log to the output lines.
//
// Notes:       - This method allocates memory for output buffer and returns
//                the pointer to it in the 'outLines' parameter
// -------------------------------------------------------------------------
int GUILog::GetLines(char* &outLines)
{
   unsigned  iLine = iFirstLine;  // Offset into circular line buffer.
   unsigned  pos   = 0;           // Offset into outLines.
   unsigned  len;                 // Length of 'lines[iLine]'.
   size_t  bufsize = 0;         // Calculate size of buffer needed
   bool      bufferfull = false;  // Loop control to prevent write past end of buffer
   string    overflowMsg = "WARNING: buffer overflow, messages truncated\n";

   myMutex.Get();

   // Calculate space required for output buffer
   for (unsigned int i = 0; i < iMaxLines; i++)
   {
      // add up the length of each string in the 'lines' vector
      bufsize += (lines[i].length() + 1);
   }
   outLines = new char[bufsize + overflowMsg.length()+1];  // allow room for overflow message (if needed)
   *outLines = '\0';

   while(iLine != iLastLine && !bufferfull)
   {
      len = lines[iLine].length();
      if(pos + len + 1 < bufsize)
      {
         strcpy (&(outLines[pos]), lines[iLine].c_str());
         outLines[pos + len] = '\n';
         len++;
         outLines[pos + len] = '\0';
         pos = pos + len;
         iLine++;
         if(iLine == iMaxLines)
         {
            iLine = 0;
         }
      }
      else
      {
         strcpy(&(outLines[pos]), overflowMsg.c_str());
         pos = pos + overflowMsg.length();
         bufferfull = true;
      }
   }

   myMutex.Release();

   return(pos + 1);
}
//GUILog::GetLines

/* -------------------------------------------------------------------------
   Name:        GUILog::PrintLine
   ------------------------------------------------------------------------- */
void GUILog::PrintLine(const std::string &asLine, bool bEOL)
{
   myMutex.Get();

   lines[iLastLine] += asLine;

   if(bEOL)
   {
      iLastLine++;

      // Adjust the circular buffer pointers if necessary.
      if (iLastLine == iMaxLines)  iLastLine = 0;
      if (iLastLine == iFirstLine) iFirstLine++;
      if (iFirstLine == iMaxLines) iFirstLine = 0;

      lines[iLastLine] = "";
   }

   if(pFnPrint)
   {
      pFnPrint(pFnPrintParam, asLine, bEOL);
   }

   myMutex.Release();
}
/*GUILog::PrintLine*/

int GUILog::GetMaxLines(void)
{
   return(iMaxLines);
}

int GUILog::GetMaxLineSize(void)
{
   return(iMaxLineSize);
}

/* -------------------------------------------------------------------------
   Name:        GUILog::WriteToFile

   Purpose:     Writes the buffer contents to the file specified.  If
                another log file is opened, it is closed at the start of
                this function and re-opened at the end of the function.
   ------------------------------------------------------------------------- */
bool GUILog::WriteToFile(const char* pszFileNm, bool bAppend, bool bUnbuffered)
{
   bool      bSuccess;
   unsigned  iLine;
   bool      bNoBuffering;          // These 3 fields used to re-open current
   bool      bAppending;            // log file.
   string previousLogName;

   myMutex.Get();

   // Don't allow the file size to exceed iMaxFileSize (in kB).
   if((fp != NULL) && (iMaxFileSize != -1) && (GetFileSize(fp) / 1024 > iMaxFileSize))
   {
      myMutex.Release();
      return(true);
   }

   // If a file is already open, it is closed and will be
   // re-opened at the end of this function.
   if(fp != NULL)
   {
      // Save information to re-open file.
      bAppending   = bFileAppend;
      bNoBuffering = bFileUnbuffered;
      previousLogName = logName;
      Close();
   }

   // Open the specified file for logging.
   Open(pszFileNm, bAppend, bUnbuffered);

   // File successfully opened.
   if(fp != NULL)
   {
      // Don't allow the file size to exceed iMaxFileSize (in kB).
      if((iMaxFileSize != -1) && (GetFileSize(fp) / 1024 > iMaxFileSize))
      {
         myMutex.Release();
         Close();
      }
      else
      {
         // Write output to log file.
         fputs ("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< START LOG "
                ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>\n", fp);

         iLine = iFirstLine;

         while(iLine != iLastLine)
         {
            fputs(lines[iLine].c_str(), fp);
            fputc('\n', fp);
            iLine++;

            if(iLine == iMaxLines)
            {
               iLine = 0;
            }
         }

         fputs ("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< END LOG "
                ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>\n", fp);
      }

      Close();
      bSuccess = true;
   }
   // Failed to open file.
   else
   {
      bSuccess = false;
   }

   // If the another file was previously open, re-open it before returning.
   if(!previousLogName.empty())
   {
      Open(previousLogName.c_str(), bAppending, bNoBuffering);
   }

   myMutex.Release();

   return(bSuccess);
}
/*GUILog::WriteToFile*/

// -------------------------------------------------------------------------
// Name:        GUILog::SaveLogToFile
//
// Purpose:     Writes a copy of the current buffer contents to the file
//              specified, ignoring settings of the GUILog object.
//              If a log file is already opened, it remains unchanged.
// -------------------------------------------------------------------------
bool GUILog::SaveLogToFile(const char* pszFileNm, bool bAppend, bool bUnbuffered)
{
   bool      bSuccess = true;           // Assume success until something goes wrong
   unsigned  iLine;
   //
   bool      previousNoBuffering;       // These 4 fields used to save and
   bool      previousAppending;         //    restore the file log properties
   string    previousLogName;
   FILE*     previousFilePointer;


   myMutex.Get();

   // Save previous property settings.
   // This method will overwrite them to perform the "save log", but then
   // will put them back when it is done.
   previousFilePointer = fp;
   previousLogName = logName;
   previousAppending   = bFileAppend;
   previousNoBuffering = bFileUnbuffered;

   fp = NULL;

   // Open the specified file for logging.
   Open(pszFileNm, bAppend, bUnbuffered);

   // File successfully opened.
   if(fp != NULL)
   {
      // Don't allow the file size to exceed iMaxFileSize (in kB).
      if((iMaxFileSize != -1) && (GetFileSize(fp) / 1024 > iMaxFileSize))
      {
         myMutex.Release();
         Close();
      }
      else
      {
         // Write output to log file.
         fputs ("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< START SAVED LOG "
                ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>\n", fp);

         iLine = iFirstLine;

         while(iLine != iLastLine)
         {
            fputs(lines[iLine].c_str(), fp);
            fputc('\n', fp);
            iLine++;

            if(iLine == iMaxLines)
            {
               iLine = 0;
            }
         }

         fputs ("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< END SAVED LOG "
                ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>\n", fp);
      }

      Close();
   }
   // Failed to open file.
   else
   {
      bSuccess = false;
   }

   // Restore the previous property settings.
   fp = previousFilePointer;
   logName = previousLogName;
   bFileAppend = previousAppending;
   bFileUnbuffered = previousNoBuffering;

   myMutex.Release();

   return(bSuccess);
}
//GUILog::SaveLogToFile

void GUILog::Reset(void)
{
	iFirstLine = iLastLine = 0;
   lines[iLastLine] = "";
}
