#include <stdarg.h>                     // va_list, etc.
#include <stdio.h>                      // printf()
#include <stdlib.h>                     // free()
#include <string.h>                     // strcmp()
#include <io.h>
#include <fcntl.h>
#include <process.h>					// _beginthreadex(), _endthread()

#include "LumenVoxVoice.h"

Voice* CreateVoice(unsigned int applid, Log* ivLog, BoardGeneric* board)
{
   return(new LumenVoxVoice(applid, ivLog, board));
}

void DestroyVoice(Voice* voiceModule)
{
   delete (LumenVoxVoice*) voiceModule;
}

LumenVoxVoice::LumenVoxVoice(unsigned int applid, Log* ivLog, BoardGeneric* board)
   : Voice(applid, ivLog, board)
{
}

LumenVoxVoice::~LumenVoxVoice(void)
{
   SRRelease();
}

static void LogCallBackFunction(const char* message, void* ivLog)
{
   if(ivLog != NULL)
   {
      ((Log*) ivLog)->Print("LumenVox: %s\n", message);
   }
}

IV_STATUS LumenVoxVoice::ParseGrammarText(const char* text)
{
   int ret;
   char concept[1024];
   char phrase[1024];
   const char* separator;
   int ivPosn;
	int ivCPosn;
   int ivPPosn;

	separator = strstr(text, "::");

   if(separator == NULL)
   {
      ivLog->Print("      ERROR: Grammar text format error\n");
      ivLog->Print("         '%s'\n", text);
      return(IVR_FAIL);
   }

   ivPosn = (int)(separator - text);

   for(ivCPosn = 0; ivCPosn < ivPosn; ivCPosn++)
   {
      concept[ivCPosn] = text[ivCPosn];
   }

   concept[ivCPosn]= '\0';

   ivPosn++;
   ivPosn++;

   for(ivPPosn = 0; text[ivPosn] > 0x0d; ivPosn++, ivPPosn++)
   {
      phrase[ivPPosn] = text[ivPosn];
	}

	phrase[ivPPosn] = '\0';

   if((ret = m_pLVSP.AddPhrase(grammarSet, concept, phrase)) == LV_FAILURE)
   {
      ivLog->Print("      ERROR: Unable to add phrases '%s' '%s'\n", concept, phrase);
		ivLog->Print("         Lumenvox port not open\n");
	   return(IVR_FAIL);
	}
	else if(ret == LV_SYSTEM_ERROR)
   {
      ivLog->Print("      ERROR: The LumenVox SRE is no longer running because the port is closed,\n");
      ivLog->Print("         or a unrecoverable SRE error has occurred\n");
      return(IVR_FAIL);
   }

   return(IVR_SUCCESS);
}

IV_STATUS LumenVoxVoice::SRAcquire(const char* srResource, int telephonyPort, const char* pszAppName,
   const char* pszLogDir, bool logSRAudio)
{
   int ret;
   int logLevel = 6;

   this->srResource = srResource != NULL ? srResource : "";
   this->telephonyPort = telephonyPort;
   this->logSRAudio = logSRAudio;
   appName = pszAppName != NULL ? pszAppName : "";
   logDir = pszLogDir != NULL ? pszLogDir : ".\\";

	//  Open Lumenvox port
	ret = m_pLVSP.OpenPort(LogCallBackFunction, ivLog, logLevel);

	if(ret == LV_FAILURE)
	{
		ivLog->Print("      ERROR: LumenVox licensing has been exceeded, there are too many ports active\n");
      return(IVR_FAIL);
	}
	else if(ret == LV_SYSTEM_ERROR)
	{
		ivLog->Print("      ERROR: The LumenVox port is already open\n");
      return(IVR_FAIL);
	}
	else if(ret != LV_SUCCESS)
	{
		ivLog->Print("      ERROR: LumenVox reported error code: %d\n", ret);
   	ivLog->Print("         '%s'\n", m_pLVSP.ReturnErrorString(ret));
      return(IVR_FAIL);
	}

	if(srResource != NULL)
	{
      std::FILE* LVParameterFile = fopen(srResource, "r");
      char currentLine[250];
      char key[250];
      char value[250];

      ivLog->Print("      Opening Lumenvox parameter file '%s'\n", srResource);

      if(LVParameterFile == NULL)
      {
         ivLog->Print("      ERROR: Lumenvox parameter file '%s' does not exist\n", srResource);
         return(IVR_FAIL);
      }

	   while(fgets(currentLine, 250, LVParameterFile) != NULL)
   	{
	   	sscanf(currentLine, "%s %s", key, value);

		   ivLog->Print("         Key '%s' value '%s'\n", key, value);

   		if(stricmp(key, "LogLevel") == 0)
	   	{
		   	logLevel = atoi(value);

			   if(logLevel > 6)
   			{
		   	   ivLog->Print("         ERROR: Lumenvox log level '%d' must be <= 6\n", logLevel);
               return(IVR_FAIL);
   			}

            m_pLVSP.RegisterAppLogMsg(LogCallBackFunction, ivLog, logLevel);
		   }
	   	else if(stricmp(key, "Language") == 0)
   		{
	   		if(stricmp(value, "English") == 0)
		   	{
            	ret = m_pLVSP.SetPropertyEx(PROP_EX_LANGUAGE, PROP_EX_VALUE_TYPE_STRING, PROP_EX_LANG_AMERICAN_ENGLISH);
	   		}
		   	else if(stricmp(value, "Spanish") == 0)
			   {
            	ret = m_pLVSP.SetPropertyEx(PROP_EX_LANGUAGE, PROP_EX_VALUE_TYPE_STRING, PROP_EX_LANG_SPANISH);
	   		}
		   	else if(stricmp(value, "French") == 0)
			   {
            	ret = m_pLVSP.SetPropertyEx(PROP_EX_LANGUAGE, PROP_EX_VALUE_TYPE_STRING, PROP_EX_LANG_FRENCH);
	   		}
			   else
		   	{
				   ivLog->Print("         ERROR: Unknown language '%s'\n", value);
               return(IVR_FAIL);
		   	}

            if(ret != LV_SUCCESS)
	         {
         		ivLog->Print("      ERROR: Unable to set decode language\n");
               return(IVR_FAIL);
         	}
   		}
         else if(stricmp(key, "Tuner") == 0)
         {
            if((ret = m_pLVSP.SetPropertyEx(PROP_EX_SAVE_SOUND_FILES, PROP_EX_VALUE_TYPE_INT, stricmp(value, "on") == 0 ? "1" : "0")) != LV_SUCCESS)
  	         {
         		ivLog->Print("      ERROR: Unable to turn %s speech tuner\n", stricmp(value, "on") == 0 ? "on" : "off");
            	ivLog->Print("         '%s'\n", m_pLVSP.ReturnErrorString(ret));
               return(IVR_FAIL);
         	}
         }
	   	else
		   {
			   ivLog->Print("         ERROR: Unknown Lumenvox Key Value pair '%s' '%s'\n", key, value);
   		}
   	}

	   fclose(LVParameterFile);
   }

   return(IVR_SUCCESS);
}

IV_STATUS LumenVoxVoice::SRAddGrammar(Voice::SR_GRAMMAR grammarType, const char* grammar)
{
   int ret;

   grammarSet++;

   switch(grammarType)
   {
      case Voice::DIGITS:
   	{
// Non-SRGS
//         ret = m_pLVSP.LoadStandardGrammar(grammarSet, GRAMMAR_DIGITS);

// SRGS
         // Based on LumenVox built in numbers grammar
         grammar = "#ABNF 1.0;\nmode voice;\nlanguage en-US;\ntag-format <lumenvox/1.0>;\n"
            "root $Digits;\n"
            "$ONE = ONE:\"1\";\n"
            "$TWO = TWO:\"2\";\n"
            "$THREE = THREE:\"3\";\n"
            "$FOUR = FOUR:\"4\";\n"
            "$FIVE = FIVE:\"5\";\n"
            "$SIX = SIX:\"6\";\n"
            "$SEVEN = SEVEN:\"7\";\n"
            "$EIGHT = EIGHT:\"8\";\n"
            "$NINE = NINE:\"9\";\n"
            "$ZERO = (/0.7/ ZERO| /0.3/ OH):\"0\";\n"
            "$Digit = ($ONE | $TWO | $THREE	| $FOUR | $FIVE | $SIX | $SEVEN | $EIGHT | $NINE | $ZERO ) {$=$$};\n"
            "$Digits = {$=\"\"} ($Digit {$+=$$})<1->;\n";

         ret = m_pLVSP.LoadGrammarFromBuffer(grammarSet, grammar);

         break;
	   }

      case Voice::YES_NO:
   	{
// Non-SRGS
/*
	   	ret = m_pLVSP.AddPhrase(grammarSet, "YES", "yes [sir]");
   		m_pLVSP.AddPhrase(grammarSet, "YES", "[That's] (Right | Correct | it)");
   		m_pLVSP.AddPhrase(grammarSet, "YES", "sure");
   		m_pLVSP.AddPhrase(grammarSet, "YES", "certainly");
   		m_pLVSP.AddPhrase(grammarSet, "YES", "of course");
   		m_pLVSP.AddPhrase(grammarSet, "YES", "yeah");
   		m_pLVSP.AddPhrase(grammarSet, "YES", "okay");
	   	m_pLVSP.AddPhrase(grammarSet, "NO", "no | now | naah");
		   m_pLVSP.AddPhrase(grammarSet, "NO", "nope");
*/

// SRGS
         grammar = "#ABNF 1.0;\nlanguage en-US;\nmode voice;\ntag-format <lumenvox/1.0>;\n"
            "root $yesno;\n"
            "$yes = (yes $GARBAGE) | yep | yeah | sure | certainly | ($GARBAGE (right | correct));\n"
            "$no = ((no | now | nah) $GARBAGE) | nope;\n"
            "$yesno = $yes{$=\"yes\";} | $no{$=\"no\";};";
         ret = m_pLVSP.LoadGrammarFromBuffer(grammarSet, grammar);

         break;
   	}

      case Voice::FILE:
   	{
// Non-SRGS
/*
         IV_STATUS status = IVR_SUCCESS;

	   	std::FILE* grammarFile = freopen(grammar, "r", stdin);
         char currentLine[1024];

   		if(grammarFile == NULL)
	   	{
		   	ivLog->Print("      ERROR: Unable to open grammar '%s'\n", grammar);
			   return(IVR_FAIL);
   		}
         else
         {
		   	ivLog->Print("      Loading grammar '%s'\n", grammar);
         }

	   	while((status == IVR_SUCCESS) && (fgets(currentLine, 1024, grammarFile) != NULL))
         {
            status = ParseGrammarText(currentLine);
         }

	   	fclose(grammarFile);
*/
// SRGS
         ret = m_pLVSP.LoadGrammar(grammarSet, grammar);

         break;
      }

      case Voice::TEXT:
      {
// Non-SRGS
/*
         IV_STATUS status = IVR_SUCCESS;
         char currentLine[1024];
         const char* lineBegin;
         const char* lineEnd;
         const char* textEnd = grammar + strlen(grammar);
         unsigned int lineLength;

         for(lineBegin = grammar; (status == IVR_SUCCESS) && (lineBegin < textEnd); lineBegin = lineEnd + 1)
         {
            lineEnd = strchr(lineBegin, '\n');
            if(lineEnd == NULL)
            {
               lineEnd = textEnd;
            }
            lineLength = lineEnd - lineBegin;
            strncpy(currentLine, lineBegin, lineLength);
            currentLine[lineLength] = '\0';
            status = ParseGrammarText(currentLine);
   		}
*/
// SRGS
         ret = m_pLVSP.LoadGrammarFromBuffer(grammarSet, grammar);

         break;
      }

      default:
      {
   		ivLog->Print("      ERROR: Unrecognised grammar type: %d\n", grammarType);
         return(IVR_FATAL);
      }
	}

   switch(ret)
   {
      case LV_SUCCESS:
      {
         break;
      }

      case LV_STANDARD_GRAMMAR_ALREADY_LOADED:
  		{
   		ivLog->Print("      ERROR: Only one LumenVox standard grammar can be loaded for a grammar set\n");
	   	return(IVR_FAIL);
  		}

      case LV_STANDARD_GRAMMAR_OUT_OF_RANGE:
	   {
		   ivLog->Print("      ERROR: The LumenVox standard grammar value is not a recognized grammar type\n");
  			return(IVR_FAIL);
   	}

      case LV_GRAMMAR_SYNTAX_WARNING:
	   {
		   ivLog->Print("      WARNING: Grammar '%s' does not follow SRGS syntax, but was understood anyway\n", grammar);
         return(IVR_SUCCESS);
   	}

      case LV_GRAMMAR_SYNTAX_ERROR:
	   {
		   ivLog->Print("      ERROR: Grammar '%s' does not follow SRGS syntax\n", grammar);
  			return(IVR_FAIL);
   	}

      case LV_GRAMMAR_LOADING_ERROR:
	   {
		   ivLog->Print("      ERROR: Unable to load grammar %s'%s'\n", grammarType == Voice::FILE ? "file " : "", grammar);
  			return(IVR_FAIL);
   	}

      default:
  		{
   		ivLog->Print("      ERROR: LumenVox reported error code: %d\n", ret);
	   	ivLog->Print("         '%s'\n", m_pLVSP.ReturnErrorString(ret));
		   return(IVR_FAIL);
  		}
   }

   if(m_pLVSP.ActivateGrammar(grammarSet) != LV_SUCCESS)
   {
	   ivLog->Print("      ERROR: Unable to activate grammar %d: '%s'\n", grammarSet, grammar);
  		return(IVR_FAIL);
   }

   return(IVR_SUCCESS);
}

IV_STATUS LumenVoxVoice::SREnd(void)
{
	return(IVR_SUCCESS);
}

IV_STATUS LumenVoxVoice::SRGetResult(char* concept, unsigned int maxLen, unsigned int& confidence)
{
   int ret;
   LumenVox::SOUND_FORMAT tfLV_Format;
   int iLV_VoiceChannel = 1;

	confidence = 0;

	if(concept == NULL)
   {
		ivLog->Print("      ERROR: NULL pointer provided for result\n");
      return(IVR_FAIL);
   }

   concept[0] = '\0';

   if(!isPendingSRAudio)
   {
		ivLog->Print("      ERROR: No pending audio to process\n");
		return(IVR_FAIL);
   }

	switch(SRAudioFormat)
   {
		case TF_WAV_8KHZ_16BIT: tfLV_Format = LumenVox::PCM_8KHZ; break;
		case TF_WAV_8KHZ_8BIT_MULAW: tfLV_Format = LumenVox::ULAW_8KHZ; break;
		case TF_WAV_8KHZ_8BIT_ALAW: tfLV_Format = LumenVox::ALAW_8KHZ; break;
	   default:
		{
			ivLog->Print("      ERROR: Unsupported sound format: %d\n", SRAudioFormat);
			return(IVR_FAIL);
		}
   }

   if(logSRAudio)
   {
      SRWriteAudioToFile(outgoingSRAudio, SRAudioLength);
   }

   if((ret = m_pLVSP.LoadVoiceChannel(iLV_VoiceChannel, (void*) outgoingSRAudio,
      SRAudioLength, tfLV_Format)) == LV_FAILURE)
	{
		ivLog->Print("      ERROR: The LumenVox sound format was specified incorrectly: %d\n", tfLV_Format);
      // Debug:
      ivLog->Print("      m_pLVSP.LoadVoiceChannel...  - @@@\n");
      ivLog->Print("         iLV_VoiceChannel = %i \n", iLV_VoiceChannel);
      ivLog->Print("         outgoingSRAudio  = 0x%x \n", outgoingSRAudio);
      ivLog->Print("         SRAudioLength    = %i \n", SRAudioLength);
      ivLog->Print("         tfLV_Format      = %i \n", tfLV_Format);
      // end debug
		return(IVR_FAIL);
	}
	else if(ret == LV_SYSTEM_ERROR)
	{
		ivLog->Print("      ERROR: The LumenVox SRE is no longer running because the port is closed,\n");
		ivLog->Print("         or a unrecoverable SRE error has occurred\n");
		return(IVR_FAIL);
	}
	else if(ret < LV_SUCCESS)
	{
		ivLog->Print("      ERROR: Unable to load voice channel\n");
		ivLog->Print("         '%s'\n", m_pLVSP.ReturnErrorString(ret));
		return(IVR_FAIL);
	}

// Non-SRGS
//   ret = m_pLVSP.Decode(iLV_VoiceChannel, LV_ACTIVE_GRAMMAR_SET, LV_DECODE_BLOCK | LV_DECODE_USE_OOV);

// SRGS
	ret = m_pLVSP.Decode(iLV_VoiceChannel, LV_ACTIVE_GRAMMAR_SET, LV_DECODE_BLOCK | LV_DECODE_SRGS_GRAMMAR | LV_DECODE_SEMANTIC_INTERPRETATION);

	if(ret < LV_SUCCESS)
	{
		ivLog->Print("      ERROR: Unable to decode: %d\n", ret);
		ivLog->Print("         '%s'\n", m_pLVSP.ReturnErrorString(ret));
		return(IVR_FAIL);
	}

   ivLog->Print("      Concepts returned: %d\n", m_pLVSP.GetNumberOfConceptsReturned(iLV_VoiceChannel));

	if(m_pLVSP.GetNumberOfInterpretations(iLV_VoiceChannel) > 0)
	{
// Non-SRGS
//		strncpy(concept, m_pLVSP.GetConcept(iLV_VoiceChannel, 0), maxLen);

// SRGS
      if(m_pLVSP.GetInterpretation(iLV_VoiceChannel, 0).ResultData().GetString() == NULL)
      {
         ivLog->Print("      WARNING: No interpretation generated\n");
         confidence = 0;
         return(IVR_SUCCESS);
      }
      else
      {
   		strncpy(concept, m_pLVSP.GetInterpretation(iLV_VoiceChannel, 0).ResultData().GetString(), maxLen);
         concept[maxLen - 1] = '\0';

         // LumenVox uses a scale of 1000, TeleFlow uses a scale of 100
	   	confidence = m_pLVSP.GetInterpretation(iLV_VoiceChannel, 0).Score() / 10;
      }
	}

   SRHeard = concept == NULL ? "" : concept;

   SREnd();

	return(IVR_SUCCESS);
}

IV_STATUS LumenVoxVoice::SRLoadGrammar(Voice::SR_GRAMMAR grammarType, const char* grammar)
{
   grammarSet = 0;

   if(m_pLVSP.DeactivateGrammars() != LV_SUCCESS)
   {
		ivLog->Print("      ERROR: Problem resetting grammars\n");
		return(IVR_FAIL);
	}

   return(SRAddGrammar(grammarType, grammar));
}

IV_STATUS LumenVoxVoice::SRRelease(void)
{
   m_pLVSP.ClosePort();
   return(IVR_SUCCESS);
}
