#ifndef ESCTIME_H
#define ESCTIME_H

#if defined (__DLL__)
#define ESCTIMEAPI __declspec(dllexport)
#else
#define ESCTIMEAPI __declspec(dllimport)
#endif

#include <winsock2.h>
#include <windows.h>
#include <string>
using std::string;

class ESCTIMEAPI ESCTime
{
   public:

      // 'Time Region' is a shortened version of time zone, where only two
      // regions are represented: GMT & local-time (where ever that happens to be)
      // When a ESCTime object is Updated(), you can specify which Region's
      // time you want assigned to it.
      enum TIME_REGION
      {
         LOCAL = 0,
         GMT = 1,
         INVALID = -1
      };

      ESCTime(void);

      // Constructor can accept date/time string (in TeleFlow format) for initialization
      // Will call the 'Assign' method, and default the 'timeRegion' property to LOCAL
      ESCTime(char* sDateTime);

      virtual ~ESCTime(void);

      // Overload assignment operator to set ESCTime to a specific date/time string,
      // in the TeleFlow format; milliseconds are optional
      // eg: "YYYY/MM/DD hh:mm:ss.ms_" or "YYYY/MM/DD hh:mm:ss"
      // To set to the current time, use the Update() method instead
      ESCTime operator=(char* sDateTime);

      // Sets this ESCTime to a specific value, supplied as a string in
      // the TeleFlow datetime format; milliseconds are optional
      // "YYYY/MM/DD hh:mm:ss.ms_" or "YYYY/MM/DD hh:mm:ss"
      // To set to the current time, use the Update() method instead
      void Assign(const char* sDateTime, TIME_REGION regionDesired = LOCAL);

      // Adds a number of seconds to the current ESCTime value
      // returns true if successful, or false if time is currently Invalid
      bool AddSeconds(int iSeconds);

      // Subtracts a number of seconds from the current ESCTime value
      // returns true if successful, or false if time is currently Invalid
      bool SubtractSeconds(int iSeconds);

      // Returns the seconds since midnight with millisecond fraction
      // Format: sssss.mmm
      // Returns -1 if invalid
      float GetSecondsSinceMidnight(void) const;

      // Returns the date
      // Format when separator is NULL: yyyy/mm/dd
      // Format when separator is not NULL: yyyy<separator>mm<separator>dd
      // Returns NULL if invalid
      const char* GetDate(const char* separator = NULL);

      // Returns day of the month: 1 to 31
      // Returns -1 if invalid
      int GetDay(void);

      // Returns day of the week: 0 (Sunday) to 6 (Saturday)
      // Returns -1 if invalid
      int GetDayOfWeek(void);

      // Returns day of the week name: Sunday, Monday, etc.
      // Returns NULL if invalid
      const char* GetDayOfWeekName(void);

      // Returns date and time in this format: yyyy-mm-dd_HH.MM.SS.ms_
      // Returns NULL if invalid
      const char* GetFileNameDate(void);

      // Returns hour of the day: 0 to 23
      // Returns -1 if invalid
      int GetHour(void);

      // Returns millisecond: 0 to 999
      // Returns -1 if invalid
      int GetMillisecond(void);

      // Returns minute of the hour: 0 to 59
      // Returns -1 if invalid
      int GetMinute(void);

      // Returns month of the year: 1 (January) to 12 (December)
      // Returns -1 if invalid
      int GetMonth(void);

      // Returns month name: January, February, etc.
      // Returns NULL if invalid
      const char* GetMonthName(void);

      int GetSecond(void);

      // Returns 3 letter day of the week name: Sun, Mon, etc.
      // Returns NULL if invalid
      const char* GetShortDayOfWeekName(void);

      // Returns 3 letter month name: Jan, Feb, etc.
      // Returns NULL if invalid
      const char* GetShortMonthName(void);

      // Returns the time
      // Format when separator is NULL: HH:MM:SS.ms_
      // Format when separator is not NULL: HH<separator>MM<separator>SS<separator>ms_
      // Returns NULL if invalid
      const char* GetTime(const char* separator = NULL, bool includeMilliseconds = true);

      // Returns the time in TeleFlow 'datetime' format, YYYY/MM/DD HH:MM:SS
      // date_separator will override default date separator; if NULL, default is "/"
      // time_separator will override default time separator; if NULL, default is ":"
      // Milliseconds are optional, and will not be provided unless includeMilliseconds = true
      // Returns NULL if invalid
      const char* GetDateTime(const char* date_separator = NULL, const char* time_separator = NULL, bool includeMilliseconds = false);

      // Returns the year
      // Returns -1 if invalid
      int GetYear(void);

      // Returns the date/time value as an HTTP-date, as described in RFC 1123
      // (example: "Tue, 15 Nov 1994 08:12:31 GMT")
      // Returns NULL if invalid
      const char* GetHTTPDateTime(void);

      // Makes IsValid return false
      void Invalidate(void);

      // Returns the difference in seconds between this ESCTime and from
      // Format: sssss.mmm
      // Uses current time if invalid
      // Returns 0 if from is invalid
      float GetSecondsSince(const ESCTime& from) const;

      // Returns the current setting of the timeRegion propery
      // A previous call to Update() method sets the Time Region at the
      // same time that it sets updates to the current time setting.
      TIME_REGION GetRegion(void) const;

      // Sets this ESCTime
      // Makes IsValid return true
      // Can set to either LOCAL time or GMT (default to local)
      float Update(TIME_REGION regionDesired = LOCAL);

      // Returns true if Update has been called since creation, or the last Invalidate
      // Returns false otherwise
      bool IsValid(void) const;

   private:

      bool valid;
      TIME_REGION timeRegion;
      SYSTEMTIME sysTime;
      string date;
      string time;
      string fileDate;
      string formatDate;
};

#endif