#include <ctype.h>	// isspace
#include <stdio.h>	// sprintf
#include <string.h>	// strlen, strcpy, strcat
#include <string>	   // STL string type
using std::string;
#include <algorithm> // transform
using std::transform;
#include "iv_str.h"

/* -------------------------------------------------------------------------
   Name:        safe_strcpy

   Purpose:     Copies the bytes from the source string to the destination
                string up to the length of the source string.  Destination
                string is always terminated.

   Returns:     The number of bytes copied from 'src' to 'dest'.  If all the
                bytes are copied, the return value is positive otherwise the
                return value will be <= 0.
   ------------------------------------------------------------------------- */
int safe_strcpy (char *dest, const char *src, int len)
{
   int  bytesCopied = 0;

   while (len && *src) {
      *dest = *src;
      dest++;
      src++;
      len--;
      bytesCopied++;
   }
   *dest = '\0';

   return *src ? -bytesCopied : bytesCopied;
}
/*safe_strcpy*/


/* -------------------------------------------------------------------------
   Name:        safe_strcat

   Purpose:     Copies the bytes from the source string to the end of the
                destination string up to the length of the source string.
                Destination string is always terminated.
   ------------------------------------------------------------------------- */
int safe_strcat (char *dest, const char *src, int len)
{
   while (len && *dest) {  // Move to the end of the string.
      dest++;
      len--;
   }

   return safe_strcpy (dest, src, len);
}
/*safe_strcat*/

/* -------------------------------------------------------------------------
   Name:        strtrim_left

   Purpose:     Removes leading whitespace from a string.

   Returns:     The length of the trimmed string.
   ------------------------------------------------------------------------- */
int strtrim_left (char *s)
{
   char  *temp;
   int    len = 0;

   if (!s || !*s)                  // Check if string passed in.
      return 0;

   temp = s;
   while (isspace (*temp))         // Find first non-space character.
      temp++;

   if (temp != s) {                // If string contains spaces, shift it left.
      while (*temp) {
         *s = *temp;
         s++;
         temp++;
         len++;
      }
      *s = '\0';
   } else
      len = strlen (s);

   return len;
}
/*strtrim_left*/

/* -------------------------------------------------------------------------
   Name:        strtrim_right

   Purpose:     Removes trailing whitespace from a string.

   Returns:     The length of the trimmed string.
   ------------------------------------------------------------------------- */
int strtrim_right (char *s)
{
   char  *temp;

   if (!s || !*s)                  // Check if string passed in.
      return 0;

   temp = s;
   while (*temp)                        // Move to the end of the string.
      temp++;

   while (temp > s && isspace(*(temp - 1)))  // Find first non-space char.
      temp--;

   *temp = '\0';

   return temp - s;
}
/*strtrim_right*/

/* -------------------------------------------------------------------------
   Name:        strtrim

   Purpose:     Removes leading and trailing whitespace from a string.

   Returns:     The length of the trimmed string.
   ------------------------------------------------------------------------- */
int strtrim (char *s)
{
   strtrim_left (s);
   return strtrim_right (s);
}
/*strtrim*/

/* -------------------------------------------------------------------------
   Name:        strclear

   Purpose:     Sets the string to the empty string.  If the string is null,
                then no memory has been allocated to the string and there is
                nothing to do (stradd and charadd will allocate memory as
                needed).
   ------------------------------------------------------------------------- */
void strclear (char *&dest)
{
   if (!dest)
      return;

   *dest = '\0';
}
/*strclear*/

/* -------------------------------------------------------------------------
   Name:        stradd

   Purpose:     Concatenates one string to another.  If the destination
                string doesn't contain enough memory to hold both strings,
                memory is allocated prior to concatenating the strings.

   Notes:       Revised to eliminate compiler warnings under CBuilder 6.
                Compiler was complaing that the integer constant 'growSize'
                was never used (when in fact it was, go figure).  Changed
                from integer constant to macro 'IV_GROWSIZE' to get rid of
                the error.  BCB6 upgrade - TF Mar.28/02
   ------------------------------------------------------------------------- */
bool stradd (
     char      *&dest
   , const char *src
   , int        &destSize
)
{
   int        srcLen;         // length of source string
   int        destLen;        // length of destination string

   if (!dest)
      destLen = 0;
   else
      destLen = strlen (dest);

   if (!src)
      srcLen = 0;
   else
      srcLen = strlen (src);

   if (destLen + srcLen >= destSize)
   {
      int        len;
      // BCB6 upgrade - TF Mar.28/02
      // const int  growSize = 64;
      char      *s;

      // Grow dest.
      len = destSize;
      while (destLen + srcLen >= len)
      {
         // BCB6 upgrade - TF Mar.28/02
         // len += growSize;
         len += IV_GROWSIZE;
      }

      s = new char [len];
      if (!s)
         return false;

      if (dest)
         strcpy (s, dest);
      else
         *s = 0;

      destSize = len;
      delete [] dest;
      dest = s;
   }

   if (src)
      strcat (dest, src);

   return true;
}
/*stradd*/

/* -------------------------------------------------------------------------
   Name:        charadd

   Purpose:     Concatenates a character to the end of a string.  If the
                destination string doesn't contain enough memory to hold the
                additional character, memory is allocated prior to
                concatenating.

   Notes:       - If appending '\0', nothing changes.
                - Revised to eliminate compiler warnings under CBuilder 6.
                  Compiler was complaing that the integer constant 'growSize'
                  was never used (when in fact it was, go figure).  Changed
                  from integer constant to macro 'IV_GROWSIZE' to get rid of
                  the error.  BCB6 upgrade - TF Mar.28/02
   ------------------------------------------------------------------------- */
bool charadd (
     char      *&dest
   , const char src
   , int        &destSize
)
{
   int        destLen;

   if (!src)
      return true;      // This function never appends an extra null character.

   if (!dest)
      destLen = 0;
   else
      destLen = strlen (dest);

   // Check to see if the destination string has enough memory to add an
   // additional character.  If not, grow the destination string.
   if (destLen + 1 >= destSize) {
      int        len;
      // BCB6 upgrade - TF Mar.28/02
      // const int  growSize = 64;
      char      *s;

      // Increase the length of the string 64 bytes at a time to prevent having
      // to re-allocate memory frequently.
      len = destSize;
      while (destLen + 1 >= len)
      {
         // BCB6 upgrade - TF Mar.28/02
         // len += growSize;
         len += IV_GROWSIZE;
      }

      // Allocate memory for the new string size.
      s = new char [len];
      if (!s)
         return false;

      // Copy the existing string to the larger memory buffer.
      if (dest)
         strcpy (s, dest);
      else
         *s = 0;

      // Point the string pointer to the larger memory buffer and release
      // the original smaller memory buffer.
      destSize = len;
      delete [] dest;
      dest = s;
   }

   // If appending a LF to a string and the previous character is a CR,
   // combine the two characters into one LF character.
   if (destLen > 0 && dest[destLen - 1] == '\r' && src == '\n')
      destLen--;

   // Append the new character to the end of the string.
   dest[destLen] = src;
   destLen++;
   dest[destLen] = 0;

   return true;
}
/*charadd*/

/* -------------------------------------------------------------------------
   Name:        GetTimeStr
   ------------------------------------------------------------------------- */
void GetTimeStr (char *&value)
{
   if (value) {
      delete [] value;
      value = 0;
   }
   value = new char [10];

   sprintf (value, "%d", time (0));
}
/*GetTimeStr*/

/* -------------------------------------------------------------------------
   Name:        GetSecondsStr

   Purpose:     Converts a time value in GMT to number of seconds since
                midnight today local time.
   ------------------------------------------------------------------------- */
void GetSecondsStr (time_t timer, char *&value)
{
   struct tm *  tBlock;
   int          seconds;

   if (value) {
      delete [] value;
      value = 0;
   }
   value = new char [10];

   tBlock  = localtime (&timer);
   seconds = tBlock->tm_hour * 3600
           + tBlock->tm_min * 60
           + tBlock->tm_sec;
   sprintf (value, "%d", seconds);
}
/*GetSecondsStr*/

std::string::size_type FindCaseInsensitiveString(const char* searchIn, const char* searchFor)
{
   string searchInString;
   string searchForString;

   if((searchIn == NULL) || (searchFor == NULL))
   {
      return(std::string::npos);
   }

   searchInString = searchIn;
   searchForString = searchFor;

   // Convert token to upper case
   transform(searchInString.begin(), searchInString.end(), searchInString.begin(), toupper);
   transform(searchForString.begin(), searchForString.end(), searchForString.begin(), toupper);

   return(searchInString.find(searchForString));
}
/*GetSecondsStr*/

/* -------------------------------------------------------------------------
   Name:        strrepl

   Purpose:     Search a C-type string (pszMain), replacing all occurrences
                of one sub-string (pszSearch) with another sub-string
                (pszReplace).

   Modified:    Jun.13/02 - TF: Created

   Notes:       Uses Standard Template Library string types

                Creates a new C-type string, and returns it's address as
                a char*
   ------------------------------------------------------------------------- */
char* strrepl (const char* pszMain, const char* pszSearch, const char* pszReplace)
{
   string             sTemp;          // Temporary STL string to perform operations on
   char*              pszRet = NULL;  // Pointer to new C-string created by this module
   string::size_type  index;          // index position for traversing sTemp
   string::size_type  len;            // string length
   string::size_type  skip;           // to skip past the newly replaced portion of the string

   if (pszMain != NULL && pszReplace != NULL)
   {
      sTemp = pszMain;

      if (pszMain[0] != '\0' && pszSearch != NULL && pszSearch[0] != '\0')
      {
         len  = strlen(pszSearch);
         skip = strlen(pszReplace);
         for(index = 0; (index = sTemp.find(pszSearch, index)) != string::npos; index += skip)
         {
            sTemp.replace(index, len, pszReplace);
         }
      }

      len = sTemp.length();
      pszRet = new char[len+1];
      strcpy(pszRet, sTemp.c_str());
   }

   return(pszRet);
}
/*strrepl*/




/* -------------------------------------------------------------------------
   Name:        FindDelimitedElement

   Purpose:     return the iOccurrence item within a delimited string

   Modified:    Sb - Created in step_parse
                2004-04-30 - JSH: moved to iv_str

   Notes:       Creates a new C-type string, and returns it's address as
                a char*
   ------------------------------------------------------------------------- */
char* FindDelimitedElement(const char* pszText, int iStartPos, int iOccurrence,
   const char* delimiter, bool bForward, bool bCaseSensitive)
{
   char* pszSearchIn;
   char* pszSearchFor;
   char* startPos;
   int delimiterLength = strlen(delimiter);
   char* endPos;
   int elementLength;
   char* result;

   if(iOccurrence < 1)
   {
      return(NULL);
   }

   pszSearchIn = new char[strlen(pszText) + 1];
   strcpy(pszSearchIn, pszText);
   pszSearchFor = new char[delimiterLength + 1];
   strcpy(pszSearchFor, delimiter);

   if(!bCaseSensitive)
   {
      strlwr(pszSearchIn);
      strlwr(pszSearchFor);
   }

   if(!bForward)
   {
      strrev(pszSearchIn);
      strrev(pszSearchFor);
      startPos = pszSearchIn + strlen(pszSearchIn) - iStartPos - 1;
   }
   else
   {
      startPos = pszSearchIn + iStartPos;
   }

   if((startPos < pszSearchIn) || (startPos >= pszSearchIn + strlen(pszSearchIn)))
   {
      delete[] pszSearchIn;
      delete[] pszSearchFor;
      return(NULL);
   }

   for(int counter = 1; (counter < iOccurrence) && (startPos != NULL); counter++)
   {
      startPos = strstr(startPos, pszSearchFor);

      if(startPos != NULL)
      {
         startPos += delimiterLength;
      }
   }

   if(startPos == NULL)
   {
      delete[] pszSearchIn;
      delete[] pszSearchFor;
      return(NULL);
   }

   endPos = strstr(startPos, pszSearchFor);

   if(endPos == NULL)
   {
      endPos = startPos + strlen(startPos);
   }

   elementLength = endPos - startPos;

   result = new char[elementLength + 1];
   if(bForward)
   {
      strncpy(result, pszText + (startPos - pszSearchIn), elementLength);
   }
   else
   {
      strncpy(result, pszText + strlen(pszText) - elementLength - (startPos - pszSearchIn), elementLength);
   }
   result[elementLength] = '\0';
   delete[] pszSearchIn;
   delete[] pszSearchFor;

   return(result);
}
/* FindDelimitedElement */

//Function to convert unsigned char to string of length 2
void Char2Hex(const unsigned char ch, char* szHex, bool uppercase = true)
{
	unsigned char byte[2];
	byte[0] = (unsigned char) (ch/16);
	byte[1] = (unsigned char) (ch%16);
	for(int i=0; i<2; i++)
	{
		if(byte[i] <= 9)
			szHex[i] = (unsigned char) ('0' + byte[i]);
		else
			szHex[i] = (unsigned char) ((uppercase ? 'A' : 'a') + byte[i] - 10);
	}
	szHex[2] = 0;
}

//Function to convert string of length 2 to unsigned char
void Hex2Char(const char* szHex, unsigned char& rch)
{
	rch = 0;
	for(int i=0; i<2; i++)
	{
		if(*(szHex + i) >='0' && *(szHex + i) <= '9')
			rch = (unsigned char) ((rch << 4) + (*(szHex + i) - '0'));
		else if(*(szHex + i) >='a' && *(szHex + i) <= 'f')
			rch = (unsigned char) ((rch << 4) + (*(szHex + i) - 'a' + 10));
		else if(*(szHex + i) >='A' && *(szHex + i) <= 'F')
			rch = (unsigned char) ((rch << 4) + (*(szHex + i) - 'A' + 10));
	}
}    

//Function to convert string of chars to string of unsigned chars
void HexStr2CharStr(const char* pszHexStr, unsigned char* pucCharStr, int iSize)
{
	int i;
	unsigned char ch;
	for(i=0; i<iSize; i++)
	{
		Hex2Char(pszHexStr+2*i, ch);
		pucCharStr[i] = ch;
	}
}

//Function to convert string of unsigned chars to string of chars
void CharStr2HexStr(const unsigned char* pucCharStr, char* pszHexStr, int iSize, bool uppercase)
{
	int i;
	char szHex[3];
	pszHexStr[0] = 0;
	for(i=0; i<iSize; i++)
	{
		Char2Hex(pucCharStr[i], szHex, uppercase);
		strcat(pszHexStr, szHex);
	}
}

// Function to remove leading and trailing spaces
// Returns a newly allocated char* which must be deleted by calling module
char* TrimStr(const char* pszInput)
{
   string tempstr;
   string::size_type pos;
   char* pszReturn;

   if(pszInput == NULL)
   {
      return(NULL);
   }

   tempstr = pszInput;

   pos = tempstr.find_last_not_of(" \t");
   if(pos != string::npos)
   {
      tempstr.erase(pos + 1);
      pos = tempstr.find_first_not_of(" \t");
      if(pos != string::npos)
      {
         tempstr.erase(0, pos);
      }
   }
   else
   {
      tempstr = "";
   }

   pszReturn = new char[tempstr.length()+1];
   if(pszReturn)
   {
      strcpy(pszReturn,tempstr.c_str());
   }

   return(pszReturn);
}

