#ifndef __SOCKET_H__
#define __SOCKET_H__

#ifdef __DLL__
#define SOCKETAPI __declspec(dllexport)
#else
#define SOCKETAPI __declspec(dllimport)
#endif

// Special warning: There is a bug in the Borland optimiser such that, if this file
// is included in a project where compiler code optimisation option is not set to
// none, errors will occur in wspiapi.h (which is included by the ACE wrapper header
// files): "Call to undefined function '___strcpy___'" or "Call to undefined function
// '___strcmp___'"

// In order to include this file you must add the ACE ACE_wrappers directory to
// your include path.

// Must be included to prevent define conflicts in windows.h
#include <winsock2.h>

#include <map>
using std::map;

#include "..\escutil\criticalsection.h"

// Warning en/disables are for Borland C++Builder, ignore in MS VC++
#ifndef _MSC_VER
// Warning disabled because the ACE code is not to our standards
#pragma warn -8061
#endif
#include "ace\SOCK_Acceptor.h"
#include "ace\SOCK_Dgram.h"
#include "ace\ssl\SSL_SOCK_Stream.h"
// Warning re-enabled because our code is
#ifndef _MSC_VER
#pragma warn +8061
#endif

class TCPSocket;

// This is a container class for socket-related functionality and is not intended
// for instantiation
class SOCKETAPI Socket
{
   public:

      enum SOCKET_ERR
      {
         // Socket functions that can return an error should return SUCCESS or a WSAError
         SUCCESS = 0
      };

      enum PORT
      {
         INVALID_PORT = 0,
         ANY_PORT = 0
      };

      virtual ~Socket(void);

      // ACE_SSL_SOCK_Stream method overwritten to distinguish between SSL and
      // non-SSL functionality
      virtual int close(void) = NULL;

      // Closes the O/S socket and clears the event handle
      // Returns Socket::SUCCESS or a WSAError
      virtual int Close(void);

      // ACE_SSL_SOCK_Stream method overwritten to distinguish between SSL and
      // non-SSL functionality
      virtual int enable(int value) const = NULL;

      // Get the underlying handle.
      virtual ACE_HANDLE get_handle(void) const = NULL;

      // ACE_SSL_SOCK_Stream method overwritten to distinguish between SSL and
      // non-SSL functionality
      virtual int get_local_addr(ACE_Addr& addr) const = NULL;

      // ACE_SSL_SOCK_Stream method overwritten to distinguish between SSL and
      // non-SSL functionality
      virtual int get_option(int level, int option, void* optval, int* optlen) const = NULL;

      // Sets events to a mask of the events that have signalled the socket event
      // handle such as FD_READ or FD_CLOSE
      // Does not set events on error
      // Returns Socket::SUCCESS, a WSAError, or an openSSL error code
      virtual int GetEvents(int& events) const;

      // Returns socket event handle
      // Returns NULL if the event handle is invalid, but the handle should always
      // be valid
		virtual HANDLE GetEventHandle(void) const;

      // Returns the IPv4 address of the host machine in "xxx.xxx.xxx.xxx" format
      static const char* GetHostAddress(void);

      // Writes host address information to host
      // Returns Socket::SUCCESS or a WSAError
      virtual int GetHostAddress(ACE_INET_Addr& host) const;

      // Returns the name of the host machine
      static const char* GetHostName(void);

      // Returns the current error code for the O/S socket
      // Returns Socket::SUCCESS, a WSAError, or an openSSL error code
      virtual int GetStatus(void) const;

      // Shuts down socket services from Winsock DLL for this process
      // Client process should only call this function once, when finished with socket services
      // Will shut down any pending socket functions
      // Returns Socket::SUCCESS or a WSAError
      static int NetShutdown(void);

      // Must be called before using socket services
      // Client process should only call this function once
      // Returns Socket::SUCCESS or a WSAError
      static int NetStartup(void);

      int SelectEvents(long lNetworkEvents) const;

      // Returns a string that corresponds to the WSAError errorCode
      // Returns "Unknown" for unknown error codes
      static const char* WSAErrorString(int errorCode);

   protected:

      bool persistant;
		HANDLE eventObject;

      // This is a container class for socket-related functionality and is not
      // intended for instantiation
      Socket(bool persistant);

      virtual int GenerateError(void) const;
};

class SOCKETAPI TCPSocket : public Socket, public ACE_SSL_SOCK_Stream
{
   public:

      // Creates a new O/S socket and an event handle
      // Sets the event handle to signal FD_READ or FD_CLOSE events for the O/S socket
      // persistant should be true if the O/S socket should not be closed by the
      // destructor
      TCPSocket(bool persistant = false);

      // Creates an event handle
      // Sets the event handle to signal FD_READ or FD_CLOSE events for the socket specified
      // by handle unless handle is ACE_INVALID_HANDLE
      TCPSocket(ACE_HANDLE handle);

      // Closes the O/S socket if persistance was set true
      // Closes the event handle
      virtual ~TCPSocket(void);

      // ACE_SSL_SOCK_Stream method overwritten to distinguish between SSL and
      // non-SSL functionality
      virtual int close(void);

      // Connects socket to the address specified by peer
      // Sets the event handle to signal FD_CONNECT events for the O/S socket
      // Connects from port specified by host, or an available port by default
      // Returns Socket::SUCCESS, a WSAError, or an openSSL error code
      virtual int Connect(const ACE_Addr& peer, const ACE_Addr& host = ACE_Addr::sap_any);

      // Sets TCPSocket to use non-SSL methods
      // This method should only be called while a TCPSocket is not connected
      // Returns Socket::SUCCESS, a WSAError, or an openSSL error code
      virtual int DisableSSL(void);

      // ACE_SSL_SOCK_Stream method overwritten to distinguish between SSL and
      // non-SSL functionality
      virtual int enable(int value) const;

      // Sets TCPSocket to use SSL methods
      // This method should only be called while a TCPSocket is not connected
      // Returns Socket::SUCCESS, a WSAError, or an openSSL error code
      virtual int EnableSSL(void);

      // Get the underlying handle.
      virtual ACE_HANDLE get_handle(void) const;

      // ACE_SSL_SOCK_Stream method overwritten to distinguish between SSL and
      // non-SSL functionality
      virtual int get_local_addr(ACE_Addr& addr) const;

      // ACE_SSL_SOCK_Stream method overwritten to distinguish between SSL and
      // non-SSL functionality
      virtual int get_option(int level, int option, void* optval, int* optlen) const;

      // ACE_SSL_SOCK_Stream method overwritten to distinguish between SSL and
      // non-SSL functionality
      virtual int get_remote_addr(ACE_Addr& addr) const;

      // Returns the number of bytes waiting to be received from the socket, and
      // includes any bytes still waiting in the SSL buffers
      // Returns Socket::SUCCESS, a WSAError, or an openSSL error code
      virtual int GetBytesWaiting(int& bytesWaiting) const;

      // Writes peer address information to peer
      // Returns Socket::SUCCESS, a WSAError, or an openSSL error code
      virtual int GetPeerAddress(ACE_Addr& peer) const;

      // Returns the pointer to the TCPSocket identified by handle
      // Returns NULL if no TCPSocket corresponds to handle
      static TCPSocket* GetSocket(ACE_HANDLE handle);

      // Returns true if in SSL mode
      // Returns false if in non-SSL mode (default)
      virtual bool IsSSLEnabled(void) const;

      // Copies up to length bytes available from the socket into a buffer
      // Sets message to a pointer to that buffer
      // The message pointer is only valid until the next receive or the object destructs
      // The number of bytes read is copied to length
      // length is set to 0 if there is no data
      // length is set to -1 if there is an error
      // Returns Socket::SUCCESS, a WSAError, or an openSSL error code
      // Returns WSAEINVAL	if length passed in is <= 0
		virtual int ReceiveBytes(const char*& message, int& length);

      // Copies the next line (text up to and including '\n' or '\0' character) available from socket
      // Sets message to a pointer to that buffer
      // The message pointer is only valid until the next receive or the object destructs
      // Returns Socket::SUCCESS, a WSAError, or an openSSL error code
      // Sets message to NULL on an error
      // Sets message to "\n" if there is no data available from the socket
		virtual int ReceiveLine(const char*& message);

      // Returns the next string (text up to '\0' character) available from socket
      // Sets message to a pointer to that buffer
      // The message pointer is only valid until the next receive or the object destructs
      // Sets message to NULL on an error
      // Sets message to "" if there is no data available from the socket
		virtual int ReceiveString(const char*& message);

      // ACE_SSL_SOCK_Stream method overwritten to distinguish between SSL and
      // non-SSL functionality
      virtual ssize_t recv(void* buf, size_t n) const;

      // ACE_SSL_SOCK_Stream method overwritten to distinguish between SSL and
      // non-SSL functionality
      virtual ssize_t recv(void* buf, size_t n, int flags) const;

      // ACE_SSL_SOCK_Stream method overwritten to distinguish between SSL and
      // non-SSL functionality
      virtual ssize_t send(const void* buf, size_t n) const;

      // Sends length bytes from message on socket
      // Client must ensure buffer is a valid buffer of at least length bytes
      // Client must ensure socket connection before use
      // Sends multiple times for lengths over the maximum packet size
      // Returns Socket::SUCCESS, a WSAError, or an openSSL error code
		virtual int SendBytes(const char* message, int length) const;

      // Sends line (text up to and including '\n' or '\0' character) on socket
      // Appends '\n' if not in message
      // Client must ensure message is a valid string
      // Client must ensure socket connection before use
      // Sends multiple times for lengths over the maximum packet size
      // Returns Socket::SUCCESS, a WSAError, or an openSSL error code
		virtual int SendLine(const char* message) const;

      // Sends string (text up to '\0' character) on socket
      // Client must ensure message is a valid string
      // Client must ensure socket connection before use
      // Sends multiple times for lengths over the maximum packet size
      // Returns Socket::SUCCESS, a WSAError, or an openSSL error code
		virtual int SendString(const char* message) const;

      // ACE_SSL_SOCK_Stream method overwritten to distinguish between SSL and
      // non-SSL functionality
      virtual void set_handle(ACE_HANDLE handle);

      // Negotiates SSL Handshake
      // This method should only be called immediately after a successful connection
      // Returns Socket::SUCCESS, a WSAError, or an openSSL error code
      virtual int SSLHandshake(void);

   protected:

      bool isSSLEnabled;
      char* buffer;

      // All TCPSockets are added to a global list when created with a valid handle, their handle is set
      // to a valid handle, or they are connected
      static void AddSocket(ACE_HANDLE handle, TCPSocket* socket);

      // All TCPSockets are removed from a global list when closed or destoyed with
      // a valid handle, or another TCPSocket is set with handle
      static TCPSocket* RemoveSocket(ACE_HANDLE handle);
};

class SOCKETAPI UDPSocket : public Socket, public ACE_SOCK_Dgram
{
   public:

      // Creates a new O/S socket on the port specified, or an available port by
      // default, and creates an event handle
      // Sets the event handle to signal FD_READ events for the O/S socket
      // persistant should be true if the O/S socket should not be closed by the
      // destructor
      UDPSocket(unsigned short port = Socket::ANY_PORT, bool persistant = false);

      // Creates an event handle
      // Sets the event handle to signal FD_READ events for the socket specified
      // by handle unless handle is ACE_INVALID_HANDLE
      // The O/S socket is not closed in the destructor
      UDPSocket(ACE_HANDLE handle);

      // Closes the O/S socket if persistance was set true
      // Closes the event handle
      virtual ~UDPSocket(void);

      // Binds the socket to the port specified, or an available port by default
      // Returns Socket::SUCCESS or a WSAError
      virtual int Bind(unsigned short port = Socket::ANY_PORT);

      // ACE_SSL_SOCK_Stream method overwritten to distinguish between SSL and
      // non-SSL functionality
      virtual int close(void);

      // ACE_SSL_SOCK_Stream method overwritten to distinguish between SSL and
      // non-SSL functionality
      virtual int enable(int value) const;

      // Get the underlying handle.
      virtual ACE_HANDLE get_handle(void) const;

      // ACE_SSL_SOCK_Stream method overwritten to distinguish between SSL and
      // non-SSL functionality
      virtual int get_local_addr(ACE_Addr& addr) const;

      // ACE_SSL_SOCK_Stream method overwritten to distinguish between SSL and
      // non-SSL functionality
      virtual int get_option(int level, int option, void* optval, int* optlen) const;

      // Returns the number of bytes waiting to be received from the socket
      // Returns Socket::SUCCESS or a WSAError
      virtual int GetBytesWaiting(int& bytesWaiting) const;

      // Moves up to length bytes currently available from socket into a buffer
      // Sets message to buffer, and message is only valid until the next receive or the object destructs
      // Sets message to NULL if there is no data, or if there is an error
      // Sets length to the number of bytes read
      // length is set to 0 if there is no data
      // length is set to -1 if there is an error
      // Returns Socket::SUCCESS or a WSAError
      // Returns WSAECONNRESET if socket is closed
      // Returns WSAEINVAL	if length passed in is <= 0
		virtual int ReceiveBytes(const char*& message, int& length, ACE_Addr& peer);

      // Moves the next line (text up to and including '\n' or '\0' character) currently available from socket into a buffer
      // Sets message to buffer, and message is only valid until the next receive or the object destructs
      // Sets message to a "\n" string if there is no data available from the socket
      // Sets message to NULL if there is an error
      // Returns Socket::SUCCESS or a WSAError
      // Returns WSAECONNRESET if socket is closed
		virtual int ReceiveLine(const char*& message, ACE_Addr& peer);

      // Moves the next string (text up to '\0' character) currently available from socket into a buffer
      // Sets message to buffer, and message is only valid until the next receive or the object destructs
      // Sets message to NULL if there is an error
      // Sets message to an empty string if there is no data available from the socket
      // Returns Socket::SUCCESS or a WSAError
      // Returns WSAECONNRESET if socket is closed
		virtual int ReceiveString(const char*& message, ACE_Addr& peer);

      // Sends length bytes from message on socket
      // Client must ensure buffer is a valid buffer of at least length bytes
      // Client must ensure socket connection before use
      // Sends multiple times for lengths over the maximum packet size
      // Returns Socket::SUCCESS or a WSAError
		virtual int SendBytes(const char* message, int length, const ACE_Addr& peer) const;

      // Sends line (text up to and including '\n' or '\0' character) on socket
      // Appends '\n' if not in message
      // Client must ensure message is a valid string
      // Client must ensure socket connection before use
      // Sends multiple times for lengths over the maximum packet size
      // Returns Socket::SUCCESS or a WSAError
		virtual int SendLine(const char* message, const ACE_Addr& peer) const;

      // Sends string (text up to '\0' character) on socket
      // Client must ensure message is a valid string
      // Client must ensure socket connection before use
      // Sends multiple times for lengths over the maximum packet size
      // Returns Socket::SUCCESS or a WSAError
		virtual int SendString(const char* message, const ACE_Addr& peer) const;

   protected:

      char* buffer;
};

class SOCKETAPI SocketServer : public Socket, public ACE_SOCK_Acceptor
{
	public:

      // Creates a new O/S socket on the port specified, or an available port by
      // default, and creates an event handle
      // Sets the event handle to signal FD_ACCEPT events for the O/S socket
      // persistant should be true if the O/S socket should not be closed by the
      // destructor
      // maxPendingConnections specifies the maximum number of connection attempts
      // that will queue on the O/S socket
		SocketServer(unsigned short port = Socket::ANY_PORT, bool persistant = false, int maxPendingConnections = ACE_DEFAULT_BACKLOG);

      // Creates an event handle
      // Sets the event handle to signal FD_ACCEPT events for the socket specified
      // by handle unless handle is ACE_INVALID_HANDLE
      // The O/S socket is not closed in the destructor
		SocketServer(ACE_HANDLE handle);

      // Closes the O/S socket if persistance was set true
      // Closes the event handle
      virtual ~SocketServer(void);

      // Connects newConnection with the next pending connection
      // Sets newConnection event handle to signal FD_READ and RD_CLOSE events for
      // the new socket
      // Returns Socket::SUCCESS or a WSAError
      virtual int Accept(TCPSocket& newConnection) const;

      // Returns the port number associated with this SocketServer
      virtual int Get_Port(void) const;

      // ACE_SSL_SOCK_Stream method overwritten to distinguish between SSL and
      // non-SSL functionality
      virtual int close(void);

      // ACE_SSL_SOCK_Stream method overwritten to distinguish between SSL and
      // non-SSL functionality
      virtual int enable(int value) const;

      // Get the underlying handle.
      virtual ACE_HANDLE get_handle(void) const;

      // ACE_SSL_SOCK_Stream method overwritten to distinguish between SSL and
      // non-SSL functionality
      virtual int get_local_addr(ACE_Addr& addr) const;

      // ACE_SSL_SOCK_Stream method overwritten to distinguish between SSL and
      // non-SSL functionality
      virtual int get_option(int level, int option, void* optval, int* optlen) const;

};

#endif
