/* -------------------------------------------------------------------------
   Rules:       - Client requests data from the server.
                - Server acknowledges all messages.
                - All messages have a header possibly followed by a data
                  block.
                - If error occurs sending the header block, close the
                  connection.
                - If error occurs receiving data, close the connection.
                - If WSAEWOULDBLOCK error occurs sending data, wait for
                  wait for write event and continue sending.
                - No new messages will be sent or received until the current
                  message has been processed and sent.
                - If data write pending, reads are discarded.
                - If a full response is not received for a request after ten
                  seconds, close the connection.

   Notes:       - There can be multiple clients connected to the server.
                  The server will create a new thread to communicate with
                  each client independently.  This code needs to be able to
                  handle multiple client connections.
   ------------------------------------------------------------------------- */
#include "inet_msg.h"
#ifdef TEST_HEAPAGNT
#include <heapagnt.h>
#endif

static void *clientData;

/* -------------------------------------------------------------------------
   Name:        NetReadMessage

   Purpose:     Reads a message from a socket.  The message may or may not
                include extra data.  If the data size is specified, this
                function waits the timeout period to attempt to read the
                data.
   ------------------------------------------------------------------------- */
bool NetReadMessage (NET_STORAGE *p, NET_MSG *event, DWORD timeout)
{
   DWORD  ret;

   // Read header.
   if (!NetRecv (p, (void*) event, sizeof (NET_MSG))) {
      PrintErrorMsg ("NetReadMessage", 0);
      return false;
   }
   event->data = 0;

   // Check if data associated with this event.  If so, read data.
   if (event->dataSize) {
      ret = WaitForSingleObject (p->hEvent, timeout);
      if (ret != WAIT_OBJECT_0) {
         return false;
      }

      if (clientData)
         delete [] clientData;
      clientData = new char [event->dataSize];
      event->data = clientData;
      if (!NetRecv (p, clientData, event->dataSize)) {
         PrintErrorMsg ("NetReadMessage", 0);
         return false;
      }
   }
   return true;
}
/*NetReadMessage*/

/* -------------------------------------------------------------------------
   Name:        NetSendMessage

   Purpose:     This function writes data to socket.

   Notes:       - May want to close the socket whenever anything unexpected
                  occurs.
   ------------------------------------------------------------------------- */
bool NetSendMessage (NET_STORAGE *p, NET_MSG *event)
{
   NET_MSG    sendEvent;

   memcpy (&sendEvent, event, sizeof (NET_MSG));
   sendEvent.data = 0;

   if(NetSend(p, (const char*) &sendEvent, sizeof(NET_MSG)) != sizeof(NET_MSG))
   {
      PrintErrorMsg ("NetSendMessage", 0);
      return false;
   }

   if (event->dataSize > 0) {
      if(NetSend(p, (const char*) event->data, event->dataSize) != event->dataSize)
      {
         PrintErrorMsg ("NetSendMessage", 0);
         return false;
      }
   }

   return true;
}
/*NetSendMessage*/

